# coding=utf-8
# *** WARNING: this file was generated by pulumigen. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from .. import _utilities
from ._enums import *

__all__ = [
    'BackgroundWorkerDetailsCreateArgs',
    'BuildFilterArgs',
    'CriteriaPropertiesCpuPropertiesArgs',
    'CriteriaPropertiesArgs',
    'CronJobDetailsCreateArgs',
    'EnvSpecificDetailsCreateArgs',
    'EnvSpecificDetailsArgs',
    'EnvVarInputArgs',
    'HeaderInputArgs',
    'ImageArgs',
    'PrivateServiceDetailsCreateArgs',
    'RegistryCredentialArgs',
    'RouteCreateArgs',
    'SecretFileInputArgs',
    'ServiceDiskArgs',
    'StaticSiteDetailsCreateArgs',
    'WebServiceDetailsCreateArgs',
    'WebServiceDetailspropertiesautoscalingCriteriaPropertiesCpuPropertiesArgs',
    'WebServiceDetailspropertiesautoscalingCriteriaPropertiesArgs',
    'WebServiceDetailspropertiesautoscalingpropertiescriteriapropertiescpuArgs',
    'WebServiceDetailspropertiesautoscalingArgs',
]

@pulumi.input_type
class BackgroundWorkerDetailsCreateArgs:
    def __init__(__self__, *,
                 runtime: pulumi.Input['BackgroundWorkerDetailsCreateRuntime'],
                 autoscaling: Optional[pulumi.Input['WebServiceDetailspropertiesautoscalingArgs']] = None,
                 disk: Optional[pulumi.Input['ServiceDiskArgs']] = None,
                 env: Optional[pulumi.Input['BackgroundWorkerDetailsCreateEnv']] = None,
                 env_specific_details: Optional[pulumi.Input['EnvSpecificDetailsCreateArgs']] = None,
                 max_shutdown_delay_seconds: Optional[pulumi.Input[int]] = None,
                 num_instances: Optional[pulumi.Input[int]] = None,
                 plan: Optional[pulumi.Input['BackgroundWorkerDetailsCreatePlan']] = None,
                 pre_deploy_command: Optional[pulumi.Input[str]] = None,
                 pull_request_previews_enabled: Optional[pulumi.Input['BackgroundWorkerDetailsCreatePullRequestPreviewsEnabled']] = None,
                 region: Optional[pulumi.Input['BackgroundWorkerDetailsCreateRegion']] = None):
        """
        :param pulumi.Input['BackgroundWorkerDetailsCreateRuntime'] runtime: Runtime
        :param pulumi.Input['BackgroundWorkerDetailsCreateEnv'] env: This field has been deprecated, runtime should be used in its place.
        :param pulumi.Input[int] max_shutdown_delay_seconds: The maximum amount of time (in seconds) that Render waits for your application process to exit gracefully after sending it a SIGTERM signal.
        :param pulumi.Input[int] num_instances: Defaults to 1
        :param pulumi.Input['BackgroundWorkerDetailsCreatePlan'] plan: Defaults to "starter"
        :param pulumi.Input['BackgroundWorkerDetailsCreatePullRequestPreviewsEnabled'] pull_request_previews_enabled: Defaults to "no"
        :param pulumi.Input['BackgroundWorkerDetailsCreateRegion'] region: Defaults to "oregon"
        """
        pulumi.set(__self__, "runtime", runtime)
        if autoscaling is not None:
            pulumi.set(__self__, "autoscaling", autoscaling)
        if disk is not None:
            pulumi.set(__self__, "disk", disk)
        if env is not None:
            pulumi.set(__self__, "env", env)
        if env_specific_details is not None:
            pulumi.set(__self__, "env_specific_details", env_specific_details)
        if max_shutdown_delay_seconds is None:
            max_shutdown_delay_seconds = 30
        if max_shutdown_delay_seconds is not None:
            pulumi.set(__self__, "max_shutdown_delay_seconds", max_shutdown_delay_seconds)
        if num_instances is None:
            num_instances = 1
        if num_instances is not None:
            pulumi.set(__self__, "num_instances", num_instances)
        if plan is None:
            plan = 'starter'
        if plan is not None:
            pulumi.set(__self__, "plan", plan)
        if pre_deploy_command is not None:
            pulumi.set(__self__, "pre_deploy_command", pre_deploy_command)
        if pull_request_previews_enabled is None:
            pull_request_previews_enabled = 'no'
        if pull_request_previews_enabled is not None:
            pulumi.set(__self__, "pull_request_previews_enabled", pull_request_previews_enabled)
        if region is None:
            region = 'oregon'
        if region is not None:
            pulumi.set(__self__, "region", region)

    @property
    @pulumi.getter
    def runtime(self) -> pulumi.Input['BackgroundWorkerDetailsCreateRuntime']:
        """
        Runtime
        """
        return pulumi.get(self, "runtime")

    @runtime.setter
    def runtime(self, value: pulumi.Input['BackgroundWorkerDetailsCreateRuntime']):
        pulumi.set(self, "runtime", value)

    @property
    @pulumi.getter
    def autoscaling(self) -> Optional[pulumi.Input['WebServiceDetailspropertiesautoscalingArgs']]:
        return pulumi.get(self, "autoscaling")

    @autoscaling.setter
    def autoscaling(self, value: Optional[pulumi.Input['WebServiceDetailspropertiesautoscalingArgs']]):
        pulumi.set(self, "autoscaling", value)

    @property
    @pulumi.getter
    def disk(self) -> Optional[pulumi.Input['ServiceDiskArgs']]:
        return pulumi.get(self, "disk")

    @disk.setter
    def disk(self, value: Optional[pulumi.Input['ServiceDiskArgs']]):
        pulumi.set(self, "disk", value)

    @property
    @pulumi.getter
    def env(self) -> Optional[pulumi.Input['BackgroundWorkerDetailsCreateEnv']]:
        """
        This field has been deprecated, runtime should be used in its place.
        """
        return pulumi.get(self, "env")

    @env.setter
    def env(self, value: Optional[pulumi.Input['BackgroundWorkerDetailsCreateEnv']]):
        pulumi.set(self, "env", value)

    @property
    @pulumi.getter(name="envSpecificDetails")
    def env_specific_details(self) -> Optional[pulumi.Input['EnvSpecificDetailsCreateArgs']]:
        return pulumi.get(self, "env_specific_details")

    @env_specific_details.setter
    def env_specific_details(self, value: Optional[pulumi.Input['EnvSpecificDetailsCreateArgs']]):
        pulumi.set(self, "env_specific_details", value)

    @property
    @pulumi.getter(name="maxShutdownDelaySeconds")
    def max_shutdown_delay_seconds(self) -> Optional[pulumi.Input[int]]:
        """
        The maximum amount of time (in seconds) that Render waits for your application process to exit gracefully after sending it a SIGTERM signal.
        """
        return pulumi.get(self, "max_shutdown_delay_seconds")

    @max_shutdown_delay_seconds.setter
    def max_shutdown_delay_seconds(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "max_shutdown_delay_seconds", value)

    @property
    @pulumi.getter(name="numInstances")
    def num_instances(self) -> Optional[pulumi.Input[int]]:
        """
        Defaults to 1
        """
        return pulumi.get(self, "num_instances")

    @num_instances.setter
    def num_instances(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "num_instances", value)

    @property
    @pulumi.getter
    def plan(self) -> Optional[pulumi.Input['BackgroundWorkerDetailsCreatePlan']]:
        """
        Defaults to "starter"
        """
        return pulumi.get(self, "plan")

    @plan.setter
    def plan(self, value: Optional[pulumi.Input['BackgroundWorkerDetailsCreatePlan']]):
        pulumi.set(self, "plan", value)

    @property
    @pulumi.getter(name="preDeployCommand")
    def pre_deploy_command(self) -> Optional[pulumi.Input[str]]:
        return pulumi.get(self, "pre_deploy_command")

    @pre_deploy_command.setter
    def pre_deploy_command(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "pre_deploy_command", value)

    @property
    @pulumi.getter(name="pullRequestPreviewsEnabled")
    def pull_request_previews_enabled(self) -> Optional[pulumi.Input['BackgroundWorkerDetailsCreatePullRequestPreviewsEnabled']]:
        """
        Defaults to "no"
        """
        return pulumi.get(self, "pull_request_previews_enabled")

    @pull_request_previews_enabled.setter
    def pull_request_previews_enabled(self, value: Optional[pulumi.Input['BackgroundWorkerDetailsCreatePullRequestPreviewsEnabled']]):
        pulumi.set(self, "pull_request_previews_enabled", value)

    @property
    @pulumi.getter
    def region(self) -> Optional[pulumi.Input['BackgroundWorkerDetailsCreateRegion']]:
        """
        Defaults to "oregon"
        """
        return pulumi.get(self, "region")

    @region.setter
    def region(self, value: Optional[pulumi.Input['BackgroundWorkerDetailsCreateRegion']]):
        pulumi.set(self, "region", value)


@pulumi.input_type
class BuildFilterArgs:
    def __init__(__self__, *,
                 ignored_paths: pulumi.Input[Sequence[pulumi.Input[str]]],
                 paths: pulumi.Input[Sequence[pulumi.Input[str]]]):
        pulumi.set(__self__, "ignored_paths", ignored_paths)
        pulumi.set(__self__, "paths", paths)

    @property
    @pulumi.getter(name="ignoredPaths")
    def ignored_paths(self) -> pulumi.Input[Sequence[pulumi.Input[str]]]:
        return pulumi.get(self, "ignored_paths")

    @ignored_paths.setter
    def ignored_paths(self, value: pulumi.Input[Sequence[pulumi.Input[str]]]):
        pulumi.set(self, "ignored_paths", value)

    @property
    @pulumi.getter
    def paths(self) -> pulumi.Input[Sequence[pulumi.Input[str]]]:
        return pulumi.get(self, "paths")

    @paths.setter
    def paths(self, value: pulumi.Input[Sequence[pulumi.Input[str]]]):
        pulumi.set(self, "paths", value)


@pulumi.input_type
class CriteriaPropertiesCpuPropertiesArgs:
    def __init__(__self__, *,
                 enabled: Optional[pulumi.Input[bool]] = None,
                 percentage: pulumi.Input[int]):
        """
        :param pulumi.Input[int] percentage: Determines when your service will be scaled. If the average resource utilization is significantly above/below the target, we will increase/decrease the number of instances.
        """
        if enabled is None:
            enabled = False
        pulumi.set(__self__, "enabled", enabled)
        pulumi.set(__self__, "percentage", percentage)

    @property
    @pulumi.getter
    def enabled(self) -> pulumi.Input[bool]:
        return pulumi.get(self, "enabled")

    @enabled.setter
    def enabled(self, value: pulumi.Input[bool]):
        pulumi.set(self, "enabled", value)

    @property
    @pulumi.getter
    def percentage(self) -> pulumi.Input[int]:
        """
        Determines when your service will be scaled. If the average resource utilization is significantly above/below the target, we will increase/decrease the number of instances.
        """
        return pulumi.get(self, "percentage")

    @percentage.setter
    def percentage(self, value: pulumi.Input[int]):
        pulumi.set(self, "percentage", value)


@pulumi.input_type
class CriteriaPropertiesArgs:
    def __init__(__self__, *,
                 cpu: pulumi.Input['CriteriaPropertiesCpuPropertiesArgs'],
                 memory: pulumi.Input['WebServiceDetailspropertiesautoscalingpropertiescriteriapropertiescpuArgs']):
        pulumi.set(__self__, "cpu", cpu)
        pulumi.set(__self__, "memory", memory)

    @property
    @pulumi.getter
    def cpu(self) -> pulumi.Input['CriteriaPropertiesCpuPropertiesArgs']:
        return pulumi.get(self, "cpu")

    @cpu.setter
    def cpu(self, value: pulumi.Input['CriteriaPropertiesCpuPropertiesArgs']):
        pulumi.set(self, "cpu", value)

    @property
    @pulumi.getter
    def memory(self) -> pulumi.Input['WebServiceDetailspropertiesautoscalingpropertiescriteriapropertiescpuArgs']:
        return pulumi.get(self, "memory")

    @memory.setter
    def memory(self, value: pulumi.Input['WebServiceDetailspropertiesautoscalingpropertiescriteriapropertiescpuArgs']):
        pulumi.set(self, "memory", value)


@pulumi.input_type
class CronJobDetailsCreateArgs:
    def __init__(__self__, *,
                 runtime: pulumi.Input['CronJobDetailsCreateRuntime'],
                 schedule: pulumi.Input[str],
                 env: Optional[pulumi.Input['CronJobDetailsCreateEnv']] = None,
                 env_specific_details: Optional[pulumi.Input['EnvSpecificDetailsArgs']] = None,
                 plan: Optional[pulumi.Input['CronJobDetailsCreatePlan']] = None,
                 region: Optional[pulumi.Input['CronJobDetailsCreateRegion']] = None):
        """
        :param pulumi.Input['CronJobDetailsCreateRuntime'] runtime: Runtime
        :param pulumi.Input['CronJobDetailsCreateEnv'] env: This field has been deprecated, runtime should be used in its place.
        :param pulumi.Input['CronJobDetailsCreatePlan'] plan: Defaults to "starter"
        :param pulumi.Input['CronJobDetailsCreateRegion'] region: Defaults to "oregon"
        """
        pulumi.set(__self__, "runtime", runtime)
        pulumi.set(__self__, "schedule", schedule)
        if env is not None:
            pulumi.set(__self__, "env", env)
        if env_specific_details is not None:
            pulumi.set(__self__, "env_specific_details", env_specific_details)
        if plan is None:
            plan = 'starter'
        if plan is not None:
            pulumi.set(__self__, "plan", plan)
        if region is None:
            region = 'oregon'
        if region is not None:
            pulumi.set(__self__, "region", region)

    @property
    @pulumi.getter
    def runtime(self) -> pulumi.Input['CronJobDetailsCreateRuntime']:
        """
        Runtime
        """
        return pulumi.get(self, "runtime")

    @runtime.setter
    def runtime(self, value: pulumi.Input['CronJobDetailsCreateRuntime']):
        pulumi.set(self, "runtime", value)

    @property
    @pulumi.getter
    def schedule(self) -> pulumi.Input[str]:
        return pulumi.get(self, "schedule")

    @schedule.setter
    def schedule(self, value: pulumi.Input[str]):
        pulumi.set(self, "schedule", value)

    @property
    @pulumi.getter
    def env(self) -> Optional[pulumi.Input['CronJobDetailsCreateEnv']]:
        """
        This field has been deprecated, runtime should be used in its place.
        """
        return pulumi.get(self, "env")

    @env.setter
    def env(self, value: Optional[pulumi.Input['CronJobDetailsCreateEnv']]):
        pulumi.set(self, "env", value)

    @property
    @pulumi.getter(name="envSpecificDetails")
    def env_specific_details(self) -> Optional[pulumi.Input['EnvSpecificDetailsArgs']]:
        return pulumi.get(self, "env_specific_details")

    @env_specific_details.setter
    def env_specific_details(self, value: Optional[pulumi.Input['EnvSpecificDetailsArgs']]):
        pulumi.set(self, "env_specific_details", value)

    @property
    @pulumi.getter
    def plan(self) -> Optional[pulumi.Input['CronJobDetailsCreatePlan']]:
        """
        Defaults to "starter"
        """
        return pulumi.get(self, "plan")

    @plan.setter
    def plan(self, value: Optional[pulumi.Input['CronJobDetailsCreatePlan']]):
        pulumi.set(self, "plan", value)

    @property
    @pulumi.getter
    def region(self) -> Optional[pulumi.Input['CronJobDetailsCreateRegion']]:
        """
        Defaults to "oregon"
        """
        return pulumi.get(self, "region")

    @region.setter
    def region(self, value: Optional[pulumi.Input['CronJobDetailsCreateRegion']]):
        pulumi.set(self, "region", value)


@pulumi.input_type
class EnvSpecificDetailsCreateArgs:
    def __init__(__self__, *,
                 build_command: Optional[pulumi.Input[str]] = None,
                 docker_command: Optional[pulumi.Input[str]] = None,
                 docker_context: Optional[pulumi.Input[str]] = None,
                 dockerfile_path: Optional[pulumi.Input[str]] = None,
                 registry_credential_id: Optional[pulumi.Input[str]] = None,
                 start_command: Optional[pulumi.Input[str]] = None):
        """
        :param pulumi.Input[str] dockerfile_path: Defaults to "./Dockerfile"
        """
        if build_command is not None:
            pulumi.set(__self__, "build_command", build_command)
        if docker_command is not None:
            pulumi.set(__self__, "docker_command", docker_command)
        if docker_context is not None:
            pulumi.set(__self__, "docker_context", docker_context)
        if dockerfile_path is not None:
            pulumi.set(__self__, "dockerfile_path", dockerfile_path)
        if registry_credential_id is not None:
            pulumi.set(__self__, "registry_credential_id", registry_credential_id)
        if start_command is not None:
            pulumi.set(__self__, "start_command", start_command)

    @property
    @pulumi.getter(name="buildCommand")
    def build_command(self) -> Optional[pulumi.Input[str]]:
        return pulumi.get(self, "build_command")

    @build_command.setter
    def build_command(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "build_command", value)

    @property
    @pulumi.getter(name="dockerCommand")
    def docker_command(self) -> Optional[pulumi.Input[str]]:
        return pulumi.get(self, "docker_command")

    @docker_command.setter
    def docker_command(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "docker_command", value)

    @property
    @pulumi.getter(name="dockerContext")
    def docker_context(self) -> Optional[pulumi.Input[str]]:
        return pulumi.get(self, "docker_context")

    @docker_context.setter
    def docker_context(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "docker_context", value)

    @property
    @pulumi.getter(name="dockerfilePath")
    def dockerfile_path(self) -> Optional[pulumi.Input[str]]:
        """
        Defaults to "./Dockerfile"
        """
        return pulumi.get(self, "dockerfile_path")

    @dockerfile_path.setter
    def dockerfile_path(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "dockerfile_path", value)

    @property
    @pulumi.getter(name="registryCredentialId")
    def registry_credential_id(self) -> Optional[pulumi.Input[str]]:
        return pulumi.get(self, "registry_credential_id")

    @registry_credential_id.setter
    def registry_credential_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "registry_credential_id", value)

    @property
    @pulumi.getter(name="startCommand")
    def start_command(self) -> Optional[pulumi.Input[str]]:
        return pulumi.get(self, "start_command")

    @start_command.setter
    def start_command(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "start_command", value)


@pulumi.input_type
class EnvSpecificDetailsArgs:
    def __init__(__self__, *,
                 build_command: Optional[pulumi.Input[str]] = None,
                 docker_command: Optional[pulumi.Input[str]] = None,
                 docker_context: Optional[pulumi.Input[str]] = None,
                 dockerfile_path: Optional[pulumi.Input[str]] = None,
                 pre_deploy_command: Optional[pulumi.Input[str]] = None,
                 registry_credential: Optional[pulumi.Input['RegistryCredentialArgs']] = None,
                 start_command: Optional[pulumi.Input[str]] = None):
        if build_command is not None:
            pulumi.set(__self__, "build_command", build_command)
        if docker_command is not None:
            pulumi.set(__self__, "docker_command", docker_command)
        if docker_context is not None:
            pulumi.set(__self__, "docker_context", docker_context)
        if dockerfile_path is not None:
            pulumi.set(__self__, "dockerfile_path", dockerfile_path)
        if pre_deploy_command is not None:
            pulumi.set(__self__, "pre_deploy_command", pre_deploy_command)
        if registry_credential is not None:
            pulumi.set(__self__, "registry_credential", registry_credential)
        if start_command is not None:
            pulumi.set(__self__, "start_command", start_command)

    @property
    @pulumi.getter(name="buildCommand")
    def build_command(self) -> Optional[pulumi.Input[str]]:
        return pulumi.get(self, "build_command")

    @build_command.setter
    def build_command(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "build_command", value)

    @property
    @pulumi.getter(name="dockerCommand")
    def docker_command(self) -> Optional[pulumi.Input[str]]:
        return pulumi.get(self, "docker_command")

    @docker_command.setter
    def docker_command(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "docker_command", value)

    @property
    @pulumi.getter(name="dockerContext")
    def docker_context(self) -> Optional[pulumi.Input[str]]:
        return pulumi.get(self, "docker_context")

    @docker_context.setter
    def docker_context(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "docker_context", value)

    @property
    @pulumi.getter(name="dockerfilePath")
    def dockerfile_path(self) -> Optional[pulumi.Input[str]]:
        return pulumi.get(self, "dockerfile_path")

    @dockerfile_path.setter
    def dockerfile_path(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "dockerfile_path", value)

    @property
    @pulumi.getter(name="preDeployCommand")
    def pre_deploy_command(self) -> Optional[pulumi.Input[str]]:
        return pulumi.get(self, "pre_deploy_command")

    @pre_deploy_command.setter
    def pre_deploy_command(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "pre_deploy_command", value)

    @property
    @pulumi.getter(name="registryCredential")
    def registry_credential(self) -> Optional[pulumi.Input['RegistryCredentialArgs']]:
        return pulumi.get(self, "registry_credential")

    @registry_credential.setter
    def registry_credential(self, value: Optional[pulumi.Input['RegistryCredentialArgs']]):
        pulumi.set(self, "registry_credential", value)

    @property
    @pulumi.getter(name="startCommand")
    def start_command(self) -> Optional[pulumi.Input[str]]:
        return pulumi.get(self, "start_command")

    @start_command.setter
    def start_command(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "start_command", value)


@pulumi.input_type
class EnvVarInputArgs:
    def __init__(__self__):
        pass


@pulumi.input_type
class HeaderInputArgs:
    def __init__(__self__, *,
                 name: pulumi.Input[str],
                 path: pulumi.Input[str],
                 value: pulumi.Input[str]):
        """
        :param pulumi.Input[str] name: Header name
        :param pulumi.Input[str] path: The request path to add the header to. Wildcards will cause headers to be applied to all matching paths.
        :param pulumi.Input[str] value: Header value
        """
        pulumi.set(__self__, "name", name)
        pulumi.set(__self__, "path", path)
        pulumi.set(__self__, "value", value)

    @property
    @pulumi.getter
    def name(self) -> pulumi.Input[str]:
        """
        Header name
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: pulumi.Input[str]):
        pulumi.set(self, "name", value)

    @property
    @pulumi.getter
    def path(self) -> pulumi.Input[str]:
        """
        The request path to add the header to. Wildcards will cause headers to be applied to all matching paths.
        """
        return pulumi.get(self, "path")

    @path.setter
    def path(self, value: pulumi.Input[str]):
        pulumi.set(self, "path", value)

    @property
    @pulumi.getter
    def value(self) -> pulumi.Input[str]:
        """
        Header value
        """
        return pulumi.get(self, "value")

    @value.setter
    def value(self, value: pulumi.Input[str]):
        pulumi.set(self, "value", value)


@pulumi.input_type
class ImageArgs:
    def __init__(__self__, *,
                 image_path: pulumi.Input[str],
                 owner_id: pulumi.Input[str],
                 registry_credential_id: Optional[pulumi.Input[str]] = None):
        """
        :param pulumi.Input[str] image_path: Path to the image used for this server (e.g docker.io/library/nginx:latest).
        :param pulumi.Input[str] owner_id: The ID of the owner for this image. This should match the owner of the service as well as the owner of any specified registry credential.
        :param pulumi.Input[str] registry_credential_id: Optional reference to the registry credential passed to the image repository to retrieve this image.
        """
        pulumi.set(__self__, "image_path", image_path)
        pulumi.set(__self__, "owner_id", owner_id)
        if registry_credential_id is not None:
            pulumi.set(__self__, "registry_credential_id", registry_credential_id)

    @property
    @pulumi.getter(name="imagePath")
    def image_path(self) -> pulumi.Input[str]:
        """
        Path to the image used for this server (e.g docker.io/library/nginx:latest).
        """
        return pulumi.get(self, "image_path")

    @image_path.setter
    def image_path(self, value: pulumi.Input[str]):
        pulumi.set(self, "image_path", value)

    @property
    @pulumi.getter(name="ownerId")
    def owner_id(self) -> pulumi.Input[str]:
        """
        The ID of the owner for this image. This should match the owner of the service as well as the owner of any specified registry credential.
        """
        return pulumi.get(self, "owner_id")

    @owner_id.setter
    def owner_id(self, value: pulumi.Input[str]):
        pulumi.set(self, "owner_id", value)

    @property
    @pulumi.getter(name="registryCredentialId")
    def registry_credential_id(self) -> Optional[pulumi.Input[str]]:
        """
        Optional reference to the registry credential passed to the image repository to retrieve this image.
        """
        return pulumi.get(self, "registry_credential_id")

    @registry_credential_id.setter
    def registry_credential_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "registry_credential_id", value)


@pulumi.input_type
class PrivateServiceDetailsCreateArgs:
    def __init__(__self__, *,
                 runtime: pulumi.Input['PrivateServiceDetailsCreateRuntime'],
                 autoscaling: Optional[pulumi.Input['WebServiceDetailspropertiesautoscalingArgs']] = None,
                 disk: Optional[pulumi.Input['ServiceDiskArgs']] = None,
                 env: Optional[pulumi.Input['PrivateServiceDetailsCreateEnv']] = None,
                 env_specific_details: Optional[pulumi.Input['EnvSpecificDetailsCreateArgs']] = None,
                 max_shutdown_delay_seconds: Optional[pulumi.Input[int]] = None,
                 num_instances: Optional[pulumi.Input[int]] = None,
                 plan: Optional[pulumi.Input['PrivateServiceDetailsCreatePlan']] = None,
                 pre_deploy_command: Optional[pulumi.Input[str]] = None,
                 pull_request_previews_enabled: Optional[pulumi.Input['PrivateServiceDetailsCreatePullRequestPreviewsEnabled']] = None,
                 region: Optional[pulumi.Input['PrivateServiceDetailsCreateRegion']] = None):
        """
        :param pulumi.Input['PrivateServiceDetailsCreateRuntime'] runtime: Runtime
        :param pulumi.Input['PrivateServiceDetailsCreateEnv'] env: This field has been deprecated, runtime should be used in its place.
        :param pulumi.Input[int] max_shutdown_delay_seconds: The maximum amount of time (in seconds) that Render waits for your application process to exit gracefully after sending it a SIGTERM signal.
        :param pulumi.Input[int] num_instances: Defaults to 1
        :param pulumi.Input['PrivateServiceDetailsCreatePlan'] plan: Defaults to "starter"
        :param pulumi.Input['PrivateServiceDetailsCreatePullRequestPreviewsEnabled'] pull_request_previews_enabled: Defaults to "no"
        :param pulumi.Input['PrivateServiceDetailsCreateRegion'] region: Defaults to "oregon"
        """
        pulumi.set(__self__, "runtime", runtime)
        if autoscaling is not None:
            pulumi.set(__self__, "autoscaling", autoscaling)
        if disk is not None:
            pulumi.set(__self__, "disk", disk)
        if env is not None:
            pulumi.set(__self__, "env", env)
        if env_specific_details is not None:
            pulumi.set(__self__, "env_specific_details", env_specific_details)
        if max_shutdown_delay_seconds is None:
            max_shutdown_delay_seconds = 30
        if max_shutdown_delay_seconds is not None:
            pulumi.set(__self__, "max_shutdown_delay_seconds", max_shutdown_delay_seconds)
        if num_instances is None:
            num_instances = 1
        if num_instances is not None:
            pulumi.set(__self__, "num_instances", num_instances)
        if plan is None:
            plan = 'starter'
        if plan is not None:
            pulumi.set(__self__, "plan", plan)
        if pre_deploy_command is not None:
            pulumi.set(__self__, "pre_deploy_command", pre_deploy_command)
        if pull_request_previews_enabled is None:
            pull_request_previews_enabled = 'no'
        if pull_request_previews_enabled is not None:
            pulumi.set(__self__, "pull_request_previews_enabled", pull_request_previews_enabled)
        if region is None:
            region = 'oregon'
        if region is not None:
            pulumi.set(__self__, "region", region)

    @property
    @pulumi.getter
    def runtime(self) -> pulumi.Input['PrivateServiceDetailsCreateRuntime']:
        """
        Runtime
        """
        return pulumi.get(self, "runtime")

    @runtime.setter
    def runtime(self, value: pulumi.Input['PrivateServiceDetailsCreateRuntime']):
        pulumi.set(self, "runtime", value)

    @property
    @pulumi.getter
    def autoscaling(self) -> Optional[pulumi.Input['WebServiceDetailspropertiesautoscalingArgs']]:
        return pulumi.get(self, "autoscaling")

    @autoscaling.setter
    def autoscaling(self, value: Optional[pulumi.Input['WebServiceDetailspropertiesautoscalingArgs']]):
        pulumi.set(self, "autoscaling", value)

    @property
    @pulumi.getter
    def disk(self) -> Optional[pulumi.Input['ServiceDiskArgs']]:
        return pulumi.get(self, "disk")

    @disk.setter
    def disk(self, value: Optional[pulumi.Input['ServiceDiskArgs']]):
        pulumi.set(self, "disk", value)

    @property
    @pulumi.getter
    def env(self) -> Optional[pulumi.Input['PrivateServiceDetailsCreateEnv']]:
        """
        This field has been deprecated, runtime should be used in its place.
        """
        return pulumi.get(self, "env")

    @env.setter
    def env(self, value: Optional[pulumi.Input['PrivateServiceDetailsCreateEnv']]):
        pulumi.set(self, "env", value)

    @property
    @pulumi.getter(name="envSpecificDetails")
    def env_specific_details(self) -> Optional[pulumi.Input['EnvSpecificDetailsCreateArgs']]:
        return pulumi.get(self, "env_specific_details")

    @env_specific_details.setter
    def env_specific_details(self, value: Optional[pulumi.Input['EnvSpecificDetailsCreateArgs']]):
        pulumi.set(self, "env_specific_details", value)

    @property
    @pulumi.getter(name="maxShutdownDelaySeconds")
    def max_shutdown_delay_seconds(self) -> Optional[pulumi.Input[int]]:
        """
        The maximum amount of time (in seconds) that Render waits for your application process to exit gracefully after sending it a SIGTERM signal.
        """
        return pulumi.get(self, "max_shutdown_delay_seconds")

    @max_shutdown_delay_seconds.setter
    def max_shutdown_delay_seconds(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "max_shutdown_delay_seconds", value)

    @property
    @pulumi.getter(name="numInstances")
    def num_instances(self) -> Optional[pulumi.Input[int]]:
        """
        Defaults to 1
        """
        return pulumi.get(self, "num_instances")

    @num_instances.setter
    def num_instances(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "num_instances", value)

    @property
    @pulumi.getter
    def plan(self) -> Optional[pulumi.Input['PrivateServiceDetailsCreatePlan']]:
        """
        Defaults to "starter"
        """
        return pulumi.get(self, "plan")

    @plan.setter
    def plan(self, value: Optional[pulumi.Input['PrivateServiceDetailsCreatePlan']]):
        pulumi.set(self, "plan", value)

    @property
    @pulumi.getter(name="preDeployCommand")
    def pre_deploy_command(self) -> Optional[pulumi.Input[str]]:
        return pulumi.get(self, "pre_deploy_command")

    @pre_deploy_command.setter
    def pre_deploy_command(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "pre_deploy_command", value)

    @property
    @pulumi.getter(name="pullRequestPreviewsEnabled")
    def pull_request_previews_enabled(self) -> Optional[pulumi.Input['PrivateServiceDetailsCreatePullRequestPreviewsEnabled']]:
        """
        Defaults to "no"
        """
        return pulumi.get(self, "pull_request_previews_enabled")

    @pull_request_previews_enabled.setter
    def pull_request_previews_enabled(self, value: Optional[pulumi.Input['PrivateServiceDetailsCreatePullRequestPreviewsEnabled']]):
        pulumi.set(self, "pull_request_previews_enabled", value)

    @property
    @pulumi.getter
    def region(self) -> Optional[pulumi.Input['PrivateServiceDetailsCreateRegion']]:
        """
        Defaults to "oregon"
        """
        return pulumi.get(self, "region")

    @region.setter
    def region(self, value: Optional[pulumi.Input['PrivateServiceDetailsCreateRegion']]):
        pulumi.set(self, "region", value)


@pulumi.input_type
class RegistryCredentialArgs:
    def __init__(__self__, *,
                 id: pulumi.Input[str],
                 name: pulumi.Input[str],
                 registry: pulumi.Input['RegistryCredentialRegistry'],
                 username: pulumi.Input[str]):
        """
        :param pulumi.Input[str] id: Unique identifier for this credential
        :param pulumi.Input[str] name: Descriptive name for this credential
        :param pulumi.Input['RegistryCredentialRegistry'] registry: The registry to use this credential with
        :param pulumi.Input[str] username: The username associated with the credential
        """
        pulumi.set(__self__, "id", id)
        pulumi.set(__self__, "name", name)
        pulumi.set(__self__, "registry", registry)
        pulumi.set(__self__, "username", username)

    @property
    @pulumi.getter
    def id(self) -> pulumi.Input[str]:
        """
        Unique identifier for this credential
        """
        return pulumi.get(self, "id")

    @id.setter
    def id(self, value: pulumi.Input[str]):
        pulumi.set(self, "id", value)

    @property
    @pulumi.getter
    def name(self) -> pulumi.Input[str]:
        """
        Descriptive name for this credential
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: pulumi.Input[str]):
        pulumi.set(self, "name", value)

    @property
    @pulumi.getter
    def registry(self) -> pulumi.Input['RegistryCredentialRegistry']:
        """
        The registry to use this credential with
        """
        return pulumi.get(self, "registry")

    @registry.setter
    def registry(self, value: pulumi.Input['RegistryCredentialRegistry']):
        pulumi.set(self, "registry", value)

    @property
    @pulumi.getter
    def username(self) -> pulumi.Input[str]:
        """
        The username associated with the credential
        """
        return pulumi.get(self, "username")

    @username.setter
    def username(self, value: pulumi.Input[str]):
        pulumi.set(self, "username", value)


@pulumi.input_type
class RouteCreateArgs:
    def __init__(__self__, *,
                 destination: pulumi.Input[str],
                 source: pulumi.Input[str],
                 type: pulumi.Input['RouteCreateType'],
                 priority: Optional[pulumi.Input[int]] = None):
        """
        :param pulumi.Input[int] priority: Redirect and Rewrite Rules are applied in priority order starting at 0. Defaults to last in the priority list.
        """
        pulumi.set(__self__, "destination", destination)
        pulumi.set(__self__, "source", source)
        pulumi.set(__self__, "type", type)
        if priority is not None:
            pulumi.set(__self__, "priority", priority)

    @property
    @pulumi.getter
    def destination(self) -> pulumi.Input[str]:
        return pulumi.get(self, "destination")

    @destination.setter
    def destination(self, value: pulumi.Input[str]):
        pulumi.set(self, "destination", value)

    @property
    @pulumi.getter
    def source(self) -> pulumi.Input[str]:
        return pulumi.get(self, "source")

    @source.setter
    def source(self, value: pulumi.Input[str]):
        pulumi.set(self, "source", value)

    @property
    @pulumi.getter
    def type(self) -> pulumi.Input['RouteCreateType']:
        return pulumi.get(self, "type")

    @type.setter
    def type(self, value: pulumi.Input['RouteCreateType']):
        pulumi.set(self, "type", value)

    @property
    @pulumi.getter
    def priority(self) -> Optional[pulumi.Input[int]]:
        """
        Redirect and Rewrite Rules are applied in priority order starting at 0. Defaults to last in the priority list.
        """
        return pulumi.get(self, "priority")

    @priority.setter
    def priority(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "priority", value)


@pulumi.input_type
class SecretFileInputArgs:
    def __init__(__self__, *,
                 content: pulumi.Input[str],
                 name: pulumi.Input[str]):
        pulumi.set(__self__, "content", content)
        pulumi.set(__self__, "name", name)

    @property
    @pulumi.getter
    def content(self) -> pulumi.Input[str]:
        return pulumi.get(self, "content")

    @content.setter
    def content(self, value: pulumi.Input[str]):
        pulumi.set(self, "content", value)

    @property
    @pulumi.getter
    def name(self) -> pulumi.Input[str]:
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: pulumi.Input[str]):
        pulumi.set(self, "name", value)


@pulumi.input_type
class ServiceDiskArgs:
    def __init__(__self__, *,
                 mount_path: pulumi.Input[str],
                 name: pulumi.Input[str],
                 size_gb: Optional[pulumi.Input[int]] = None):
        """
        :param pulumi.Input[int] size_gb: Defaults to 1
        """
        pulumi.set(__self__, "mount_path", mount_path)
        pulumi.set(__self__, "name", name)
        if size_gb is not None:
            pulumi.set(__self__, "size_gb", size_gb)

    @property
    @pulumi.getter(name="mountPath")
    def mount_path(self) -> pulumi.Input[str]:
        return pulumi.get(self, "mount_path")

    @mount_path.setter
    def mount_path(self, value: pulumi.Input[str]):
        pulumi.set(self, "mount_path", value)

    @property
    @pulumi.getter
    def name(self) -> pulumi.Input[str]:
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: pulumi.Input[str]):
        pulumi.set(self, "name", value)

    @property
    @pulumi.getter(name="sizeGB")
    def size_gb(self) -> Optional[pulumi.Input[int]]:
        """
        Defaults to 1
        """
        return pulumi.get(self, "size_gb")

    @size_gb.setter
    def size_gb(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "size_gb", value)


@pulumi.input_type
class StaticSiteDetailsCreateArgs:
    def __init__(__self__, *,
                 build_command: Optional[pulumi.Input[str]] = None,
                 headers: Optional[pulumi.Input[Sequence[pulumi.Input['HeaderInputArgs']]]] = None,
                 publish_path: Optional[pulumi.Input[str]] = None,
                 pull_request_previews_enabled: Optional[pulumi.Input['StaticSiteDetailsCreatePullRequestPreviewsEnabled']] = None,
                 routes: Optional[pulumi.Input[Sequence[pulumi.Input['RouteCreateArgs']]]] = None):
        """
        :param pulumi.Input[str] publish_path: Defaults to "public"
        :param pulumi.Input['StaticSiteDetailsCreatePullRequestPreviewsEnabled'] pull_request_previews_enabled: Defaults to "no"
        """
        if build_command is not None:
            pulumi.set(__self__, "build_command", build_command)
        if headers is not None:
            pulumi.set(__self__, "headers", headers)
        if publish_path is not None:
            pulumi.set(__self__, "publish_path", publish_path)
        if pull_request_previews_enabled is None:
            pull_request_previews_enabled = 'no'
        if pull_request_previews_enabled is not None:
            pulumi.set(__self__, "pull_request_previews_enabled", pull_request_previews_enabled)
        if routes is not None:
            pulumi.set(__self__, "routes", routes)

    @property
    @pulumi.getter(name="buildCommand")
    def build_command(self) -> Optional[pulumi.Input[str]]:
        return pulumi.get(self, "build_command")

    @build_command.setter
    def build_command(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "build_command", value)

    @property
    @pulumi.getter
    def headers(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['HeaderInputArgs']]]]:
        return pulumi.get(self, "headers")

    @headers.setter
    def headers(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['HeaderInputArgs']]]]):
        pulumi.set(self, "headers", value)

    @property
    @pulumi.getter(name="publishPath")
    def publish_path(self) -> Optional[pulumi.Input[str]]:
        """
        Defaults to "public"
        """
        return pulumi.get(self, "publish_path")

    @publish_path.setter
    def publish_path(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "publish_path", value)

    @property
    @pulumi.getter(name="pullRequestPreviewsEnabled")
    def pull_request_previews_enabled(self) -> Optional[pulumi.Input['StaticSiteDetailsCreatePullRequestPreviewsEnabled']]:
        """
        Defaults to "no"
        """
        return pulumi.get(self, "pull_request_previews_enabled")

    @pull_request_previews_enabled.setter
    def pull_request_previews_enabled(self, value: Optional[pulumi.Input['StaticSiteDetailsCreatePullRequestPreviewsEnabled']]):
        pulumi.set(self, "pull_request_previews_enabled", value)

    @property
    @pulumi.getter
    def routes(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['RouteCreateArgs']]]]:
        return pulumi.get(self, "routes")

    @routes.setter
    def routes(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['RouteCreateArgs']]]]):
        pulumi.set(self, "routes", value)


@pulumi.input_type
class WebServiceDetailsCreateArgs:
    def __init__(__self__, *,
                 runtime: pulumi.Input['WebServiceDetailsCreateRuntime'],
                 autoscaling: Optional[pulumi.Input['WebServiceDetailspropertiesautoscalingArgs']] = None,
                 disk: Optional[pulumi.Input['ServiceDiskArgs']] = None,
                 env: Optional[pulumi.Input['WebServiceDetailsCreateEnv']] = None,
                 env_specific_details: Optional[pulumi.Input['EnvSpecificDetailsCreateArgs']] = None,
                 health_check_path: Optional[pulumi.Input[str]] = None,
                 max_shutdown_delay_seconds: Optional[pulumi.Input[int]] = None,
                 num_instances: Optional[pulumi.Input[int]] = None,
                 plan: Optional[pulumi.Input['WebServiceDetailsCreatePlan']] = None,
                 pre_deploy_command: Optional[pulumi.Input[str]] = None,
                 pull_request_previews_enabled: Optional[pulumi.Input['WebServiceDetailsCreatePullRequestPreviewsEnabled']] = None,
                 region: Optional[pulumi.Input['WebServiceDetailsCreateRegion']] = None):
        """
        :param pulumi.Input['WebServiceDetailsCreateRuntime'] runtime: Runtime
        :param pulumi.Input['WebServiceDetailsCreateEnv'] env: This field has been deprecated, runtime should be used in its place.
        :param pulumi.Input[int] max_shutdown_delay_seconds: The maximum amount of time (in seconds) that Render waits for your application process to exit gracefully after sending it a SIGTERM signal.
        :param pulumi.Input[int] num_instances: Defaults to 1
        :param pulumi.Input['WebServiceDetailsCreatePlan'] plan: Defaults to "starter"
        :param pulumi.Input['WebServiceDetailsCreatePullRequestPreviewsEnabled'] pull_request_previews_enabled: Defaults to "no"
        :param pulumi.Input['WebServiceDetailsCreateRegion'] region: Defaults to "oregon"
        """
        pulumi.set(__self__, "runtime", runtime)
        if autoscaling is not None:
            pulumi.set(__self__, "autoscaling", autoscaling)
        if disk is not None:
            pulumi.set(__self__, "disk", disk)
        if env is not None:
            pulumi.set(__self__, "env", env)
        if env_specific_details is not None:
            pulumi.set(__self__, "env_specific_details", env_specific_details)
        if health_check_path is not None:
            pulumi.set(__self__, "health_check_path", health_check_path)
        if max_shutdown_delay_seconds is None:
            max_shutdown_delay_seconds = 30
        if max_shutdown_delay_seconds is not None:
            pulumi.set(__self__, "max_shutdown_delay_seconds", max_shutdown_delay_seconds)
        if num_instances is not None:
            pulumi.set(__self__, "num_instances", num_instances)
        if plan is None:
            plan = 'starter'
        if plan is not None:
            pulumi.set(__self__, "plan", plan)
        if pre_deploy_command is not None:
            pulumi.set(__self__, "pre_deploy_command", pre_deploy_command)
        if pull_request_previews_enabled is None:
            pull_request_previews_enabled = 'no'
        if pull_request_previews_enabled is not None:
            pulumi.set(__self__, "pull_request_previews_enabled", pull_request_previews_enabled)
        if region is None:
            region = 'oregon'
        if region is not None:
            pulumi.set(__self__, "region", region)

    @property
    @pulumi.getter
    def runtime(self) -> pulumi.Input['WebServiceDetailsCreateRuntime']:
        """
        Runtime
        """
        return pulumi.get(self, "runtime")

    @runtime.setter
    def runtime(self, value: pulumi.Input['WebServiceDetailsCreateRuntime']):
        pulumi.set(self, "runtime", value)

    @property
    @pulumi.getter
    def autoscaling(self) -> Optional[pulumi.Input['WebServiceDetailspropertiesautoscalingArgs']]:
        return pulumi.get(self, "autoscaling")

    @autoscaling.setter
    def autoscaling(self, value: Optional[pulumi.Input['WebServiceDetailspropertiesautoscalingArgs']]):
        pulumi.set(self, "autoscaling", value)

    @property
    @pulumi.getter
    def disk(self) -> Optional[pulumi.Input['ServiceDiskArgs']]:
        return pulumi.get(self, "disk")

    @disk.setter
    def disk(self, value: Optional[pulumi.Input['ServiceDiskArgs']]):
        pulumi.set(self, "disk", value)

    @property
    @pulumi.getter
    def env(self) -> Optional[pulumi.Input['WebServiceDetailsCreateEnv']]:
        """
        This field has been deprecated, runtime should be used in its place.
        """
        return pulumi.get(self, "env")

    @env.setter
    def env(self, value: Optional[pulumi.Input['WebServiceDetailsCreateEnv']]):
        pulumi.set(self, "env", value)

    @property
    @pulumi.getter(name="envSpecificDetails")
    def env_specific_details(self) -> Optional[pulumi.Input['EnvSpecificDetailsCreateArgs']]:
        return pulumi.get(self, "env_specific_details")

    @env_specific_details.setter
    def env_specific_details(self, value: Optional[pulumi.Input['EnvSpecificDetailsCreateArgs']]):
        pulumi.set(self, "env_specific_details", value)

    @property
    @pulumi.getter(name="healthCheckPath")
    def health_check_path(self) -> Optional[pulumi.Input[str]]:
        return pulumi.get(self, "health_check_path")

    @health_check_path.setter
    def health_check_path(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "health_check_path", value)

    @property
    @pulumi.getter(name="maxShutdownDelaySeconds")
    def max_shutdown_delay_seconds(self) -> Optional[pulumi.Input[int]]:
        """
        The maximum amount of time (in seconds) that Render waits for your application process to exit gracefully after sending it a SIGTERM signal.
        """
        return pulumi.get(self, "max_shutdown_delay_seconds")

    @max_shutdown_delay_seconds.setter
    def max_shutdown_delay_seconds(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "max_shutdown_delay_seconds", value)

    @property
    @pulumi.getter(name="numInstances")
    def num_instances(self) -> Optional[pulumi.Input[int]]:
        """
        Defaults to 1
        """
        return pulumi.get(self, "num_instances")

    @num_instances.setter
    def num_instances(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "num_instances", value)

    @property
    @pulumi.getter
    def plan(self) -> Optional[pulumi.Input['WebServiceDetailsCreatePlan']]:
        """
        Defaults to "starter"
        """
        return pulumi.get(self, "plan")

    @plan.setter
    def plan(self, value: Optional[pulumi.Input['WebServiceDetailsCreatePlan']]):
        pulumi.set(self, "plan", value)

    @property
    @pulumi.getter(name="preDeployCommand")
    def pre_deploy_command(self) -> Optional[pulumi.Input[str]]:
        return pulumi.get(self, "pre_deploy_command")

    @pre_deploy_command.setter
    def pre_deploy_command(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "pre_deploy_command", value)

    @property
    @pulumi.getter(name="pullRequestPreviewsEnabled")
    def pull_request_previews_enabled(self) -> Optional[pulumi.Input['WebServiceDetailsCreatePullRequestPreviewsEnabled']]:
        """
        Defaults to "no"
        """
        return pulumi.get(self, "pull_request_previews_enabled")

    @pull_request_previews_enabled.setter
    def pull_request_previews_enabled(self, value: Optional[pulumi.Input['WebServiceDetailsCreatePullRequestPreviewsEnabled']]):
        pulumi.set(self, "pull_request_previews_enabled", value)

    @property
    @pulumi.getter
    def region(self) -> Optional[pulumi.Input['WebServiceDetailsCreateRegion']]:
        """
        Defaults to "oregon"
        """
        return pulumi.get(self, "region")

    @region.setter
    def region(self, value: Optional[pulumi.Input['WebServiceDetailsCreateRegion']]):
        pulumi.set(self, "region", value)


@pulumi.input_type
class WebServiceDetailspropertiesautoscalingCriteriaPropertiesCpuPropertiesArgs:
    def __init__(__self__, *,
                 enabled: Optional[pulumi.Input[bool]] = None,
                 percentage: pulumi.Input[int]):
        """
        :param pulumi.Input[int] percentage: Determines when your service will be scaled. If the average resource utilization is significantly above/below the target, we will increase/decrease the number of instances.
        """
        if enabled is None:
            enabled = False
        pulumi.set(__self__, "enabled", enabled)
        pulumi.set(__self__, "percentage", percentage)

    @property
    @pulumi.getter
    def enabled(self) -> pulumi.Input[bool]:
        return pulumi.get(self, "enabled")

    @enabled.setter
    def enabled(self, value: pulumi.Input[bool]):
        pulumi.set(self, "enabled", value)

    @property
    @pulumi.getter
    def percentage(self) -> pulumi.Input[int]:
        """
        Determines when your service will be scaled. If the average resource utilization is significantly above/below the target, we will increase/decrease the number of instances.
        """
        return pulumi.get(self, "percentage")

    @percentage.setter
    def percentage(self, value: pulumi.Input[int]):
        pulumi.set(self, "percentage", value)


@pulumi.input_type
class WebServiceDetailspropertiesautoscalingCriteriaPropertiesArgs:
    def __init__(__self__, *,
                 cpu: pulumi.Input['WebServiceDetailspropertiesautoscalingCriteriaPropertiesCpuPropertiesArgs'],
                 memory: pulumi.Input['WebServiceDetailspropertiesautoscalingpropertiescriteriapropertiescpuArgs']):
        pulumi.set(__self__, "cpu", cpu)
        pulumi.set(__self__, "memory", memory)

    @property
    @pulumi.getter
    def cpu(self) -> pulumi.Input['WebServiceDetailspropertiesautoscalingCriteriaPropertiesCpuPropertiesArgs']:
        return pulumi.get(self, "cpu")

    @cpu.setter
    def cpu(self, value: pulumi.Input['WebServiceDetailspropertiesautoscalingCriteriaPropertiesCpuPropertiesArgs']):
        pulumi.set(self, "cpu", value)

    @property
    @pulumi.getter
    def memory(self) -> pulumi.Input['WebServiceDetailspropertiesautoscalingpropertiescriteriapropertiescpuArgs']:
        return pulumi.get(self, "memory")

    @memory.setter
    def memory(self, value: pulumi.Input['WebServiceDetailspropertiesautoscalingpropertiescriteriapropertiescpuArgs']):
        pulumi.set(self, "memory", value)


@pulumi.input_type
class WebServiceDetailspropertiesautoscalingpropertiescriteriapropertiescpuArgs:
    def __init__(__self__, *,
                 enabled: Optional[pulumi.Input[bool]] = None,
                 percentage: pulumi.Input[int]):
        """
        :param pulumi.Input[int] percentage: Determines when your service will be scaled. If the average resource utilization is significantly above/below the target, we will increase/decrease the number of instances.
        """
        if enabled is None:
            enabled = False
        pulumi.set(__self__, "enabled", enabled)
        pulumi.set(__self__, "percentage", percentage)

    @property
    @pulumi.getter
    def enabled(self) -> pulumi.Input[bool]:
        return pulumi.get(self, "enabled")

    @enabled.setter
    def enabled(self, value: pulumi.Input[bool]):
        pulumi.set(self, "enabled", value)

    @property
    @pulumi.getter
    def percentage(self) -> pulumi.Input[int]:
        """
        Determines when your service will be scaled. If the average resource utilization is significantly above/below the target, we will increase/decrease the number of instances.
        """
        return pulumi.get(self, "percentage")

    @percentage.setter
    def percentage(self, value: pulumi.Input[int]):
        pulumi.set(self, "percentage", value)


@pulumi.input_type
class WebServiceDetailspropertiesautoscalingArgs:
    def __init__(__self__, *,
                 criteria: pulumi.Input['WebServiceDetailspropertiesautoscalingCriteriaPropertiesArgs'],
                 enabled: Optional[pulumi.Input[bool]] = None,
                 max: pulumi.Input[int],
                 min: pulumi.Input[int]):
        """
        :param pulumi.Input[int] max: The maximum number of instances for the service
        :param pulumi.Input[int] min: The minimum number of instances for the service
        """
        pulumi.set(__self__, "criteria", criteria)
        if enabled is None:
            enabled = False
        pulumi.set(__self__, "enabled", enabled)
        pulumi.set(__self__, "max", max)
        pulumi.set(__self__, "min", min)

    @property
    @pulumi.getter
    def criteria(self) -> pulumi.Input['WebServiceDetailspropertiesautoscalingCriteriaPropertiesArgs']:
        return pulumi.get(self, "criteria")

    @criteria.setter
    def criteria(self, value: pulumi.Input['WebServiceDetailspropertiesautoscalingCriteriaPropertiesArgs']):
        pulumi.set(self, "criteria", value)

    @property
    @pulumi.getter
    def enabled(self) -> pulumi.Input[bool]:
        return pulumi.get(self, "enabled")

    @enabled.setter
    def enabled(self, value: pulumi.Input[bool]):
        pulumi.set(self, "enabled", value)

    @property
    @pulumi.getter
    def max(self) -> pulumi.Input[int]:
        """
        The maximum number of instances for the service
        """
        return pulumi.get(self, "max")

    @max.setter
    def max(self, value: pulumi.Input[int]):
        pulumi.set(self, "max", value)

    @property
    @pulumi.getter
    def min(self) -> pulumi.Input[int]:
        """
        The minimum number of instances for the service
        """
        return pulumi.get(self, "min")

    @min.setter
    def min(self, value: pulumi.Input[int]):
        pulumi.set(self, "min", value)





# coding=utf-8
# *** WARNING: this file was generated by pulumigen. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload, Awaitable
from .. import _utilities

__all__ = [
    'PostgresConnectionInfo',
    'AwaitablePostgresConnectionInfo',
    'get_postgres_connection_info',
    'get_postgres_connection_info_output',
]

@pulumi.output_type
class PostgresConnectionInfo:
    def __init__(__self__, external_connection_string=None, internal_connection_string=None, password=None, psql_command=None):
        if external_connection_string and not isinstance(external_connection_string, str):
            raise TypeError("Expected argument 'external_connection_string' to be a str")
        pulumi.set(__self__, "external_connection_string", external_connection_string)
        if internal_connection_string and not isinstance(internal_connection_string, str):
            raise TypeError("Expected argument 'internal_connection_string' to be a str")
        pulumi.set(__self__, "internal_connection_string", internal_connection_string)
        if password and not isinstance(password, str):
            raise TypeError("Expected argument 'password' to be a str")
        pulumi.set(__self__, "password", password)
        if psql_command and not isinstance(psql_command, str):
            raise TypeError("Expected argument 'psql_command' to be a str")
        pulumi.set(__self__, "psql_command", psql_command)

    @property
    @pulumi.getter(name="externalConnectionString")
    def external_connection_string(self) -> str:
        return pulumi.get(self, "external_connection_string")

    @property
    @pulumi.getter(name="internalConnectionString")
    def internal_connection_string(self) -> str:
        return pulumi.get(self, "internal_connection_string")

    @property
    @pulumi.getter
    def password(self) -> str:
        return pulumi.get(self, "password")

    @property
    @pulumi.getter(name="psqlCommand")
    def psql_command(self) -> str:
        return pulumi.get(self, "psql_command")


class AwaitablePostgresConnectionInfo(PostgresConnectionInfo):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return PostgresConnectionInfo(
            external_connection_string=self.external_connection_string,
            internal_connection_string=self.internal_connection_string,
            password=self.password,
            psql_command=self.psql_command)


def get_postgres_connection_info(postgres_id: Optional[str] = None,
                                 opts: Optional[pulumi.InvokeOptions] = None) -> AwaitablePostgresConnectionInfo:
    """
    Use this data source to access information about an existing resource.
    """
    __args__ = dict()
    __args__['postgresId'] = postgres_id
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('render:postgres:getPostgresConnectionInfo', __args__, opts=opts, typ=PostgresConnectionInfo).value

    return AwaitablePostgresConnectionInfo(
        external_connection_string=pulumi.get(__ret__, 'external_connection_string'),
        internal_connection_string=pulumi.get(__ret__, 'internal_connection_string'),
        password=pulumi.get(__ret__, 'password'),
        psql_command=pulumi.get(__ret__, 'psql_command'))


@_utilities.lift_output_func(get_postgres_connection_info)
def get_postgres_connection_info_output(postgres_id: Optional[pulumi.Input[str]] = None,
                                        opts: Optional[pulumi.InvokeOptions] = None) -> pulumi.Output[PostgresConnectionInfo]:
    """
    Use this data source to access information about an existing resource.
    """
    ...



# coding=utf-8
# *** WARNING: this file was generated by pulumigen. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload, Awaitable
from .. import _utilities
from . import outputs
from ._enums import *

__all__ = [
    'CustomDomain',
    'AwaitableCustomDomain',
    'get_custom_domain',
    'get_custom_domain_output',
]

@pulumi.output_type
class CustomDomain:
    def __init__(__self__, created_at=None, domain_type=None, id=None, name=None, public_suffix=None, redirect_for_name=None, server=None, verification_status=None):
        if created_at and not isinstance(created_at, str):
            raise TypeError("Expected argument 'created_at' to be a str")
        pulumi.set(__self__, "created_at", created_at)
        if domain_type and not isinstance(domain_type, str):
            raise TypeError("Expected argument 'domain_type' to be a str")
        pulumi.set(__self__, "domain_type", domain_type)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if name and not isinstance(name, str):
            raise TypeError("Expected argument 'name' to be a str")
        pulumi.set(__self__, "name", name)
        if public_suffix and not isinstance(public_suffix, str):
            raise TypeError("Expected argument 'public_suffix' to be a str")
        pulumi.set(__self__, "public_suffix", public_suffix)
        if redirect_for_name and not isinstance(redirect_for_name, str):
            raise TypeError("Expected argument 'redirect_for_name' to be a str")
        pulumi.set(__self__, "redirect_for_name", redirect_for_name)
        if server and not isinstance(server, dict):
            raise TypeError("Expected argument 'server' to be a dict")
        pulumi.set(__self__, "server", server)
        if verification_status and not isinstance(verification_status, str):
            raise TypeError("Expected argument 'verification_status' to be a str")
        pulumi.set(__self__, "verification_status", verification_status)

    @property
    @pulumi.getter(name="createdAt")
    def created_at(self) -> str:
        return pulumi.get(self, "created_at")

    @property
    @pulumi.getter(name="domainType")
    def domain_type(self) -> 'CustomDomainDomainType':
        return pulumi.get(self, "domain_type")

    @property
    @pulumi.getter
    def id(self) -> str:
        return pulumi.get(self, "id")

    @property
    @pulumi.getter
    def name(self) -> str:
        return pulumi.get(self, "name")

    @property
    @pulumi.getter(name="publicSuffix")
    def public_suffix(self) -> str:
        return pulumi.get(self, "public_suffix")

    @property
    @pulumi.getter(name="redirectForName")
    def redirect_for_name(self) -> str:
        return pulumi.get(self, "redirect_for_name")

    @property
    @pulumi.getter
    def server(self) -> Optional['outputs.CustomDomainServerProperties']:
        return pulumi.get(self, "server")

    @property
    @pulumi.getter(name="verificationStatus")
    def verification_status(self) -> 'CustomDomainVerificationStatus':
        return pulumi.get(self, "verification_status")


class AwaitableCustomDomain(CustomDomain):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return CustomDomain(
            created_at=self.created_at,
            domain_type=self.domain_type,
            id=self.id,
            name=self.name,
            public_suffix=self.public_suffix,
            redirect_for_name=self.redirect_for_name,
            server=self.server,
            verification_status=self.verification_status)


def get_custom_domain(custom_domain_id_or_name: Optional[str] = None,
                      service_id: Optional[str] = None,
                      opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableCustomDomain:
    """
    Use this data source to access information about an existing resource.

    :param str custom_domain_id_or_name: The ID or name of the custom domain
    :param str service_id: The ID of the service
    """
    __args__ = dict()
    __args__['customDomainIdOrName'] = custom_domain_id_or_name
    __args__['serviceId'] = service_id
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('render:services:getCustomDomain', __args__, opts=opts, typ=CustomDomain).value

    return AwaitableCustomDomain(
        created_at=pulumi.get(__ret__, 'created_at'),
        domain_type=pulumi.get(__ret__, 'domain_type'),
        id=pulumi.get(__ret__, 'id'),
        name=pulumi.get(__ret__, 'name'),
        public_suffix=pulumi.get(__ret__, 'public_suffix'),
        redirect_for_name=pulumi.get(__ret__, 'redirect_for_name'),
        server=pulumi.get(__ret__, 'server'),
        verification_status=pulumi.get(__ret__, 'verification_status'))


@_utilities.lift_output_func(get_custom_domain)
def get_custom_domain_output(custom_domain_id_or_name: Optional[pulumi.Input[str]] = None,
                             service_id: Optional[pulumi.Input[str]] = None,
                             opts: Optional[pulumi.InvokeOptions] = None) -> pulumi.Output[CustomDomain]:
    """
    Use this data source to access information about an existing resource.

    :param str custom_domain_id_or_name: The ID or name of the custom domain
    :param str service_id: The ID of the service
    """
    ...



# coding=utf-8
# *** WARNING: this file was generated by pulumigen. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from .. import _utilities
from . import outputs
from ._enums import *

__all__ = [
    'ListDeploysResult',
    'AwaitableListDeploysResult',
    'list_deploys',
    'list_deploys_output',
]

@pulumi.output_type
class ListDeploysResult:
    def __init__(__self__, items=None):
        if items and not isinstance(items, list):
            raise TypeError("Expected argument 'items' to be a list")
        pulumi.set(__self__, "items", items)

    @property
    @pulumi.getter
    def items(self) -> Sequence['outputs.ListDeploysItemProperties']:
        return pulumi.get(self, "items")


class AwaitableListDeploysResult(ListDeploysResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return ListDeploysResult(
            items=self.items)


def list_deploys(service_id: Optional[str] = None,
                 opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableListDeploysResult:
    """
    Use this data source to access information about an existing resource.

    :param str service_id: The ID of the service
    """
    __args__ = dict()
    __args__['serviceId'] = service_id
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('render:services:listDeploys', __args__, opts=opts, typ=ListDeploysResult).value

    return AwaitableListDeploysResult(
        items=pulumi.get(__ret__, 'items'))


@_utilities.lift_output_func(list_deploys)
def list_deploys_output(service_id: Optional[pulumi.Input[str]] = None,
                        opts: Optional[pulumi.InvokeOptions] = None) -> pulumi.Output[ListDeploysResult]:
    """
    Use this data source to access information about an existing resource.

    :param str service_id: The ID of the service
    """
    ...



# coding=utf-8
# *** WARNING: this file was generated by pulumigen. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

from enum import Enum

__all__ = [
    'GetOwnerNotificationSettingPropertiesNotificationsToSend',
    'GetServiceNotificationOverridePropertiesNotificationsToSend',
    'GetServiceNotificationOverridePropertiesPreviewNotificationsEnabled',
    'NotificationOverrideWithCursorOverridePropertiesNotificationsToSend',
    'NotificationOverrideWithCursorOverridePropertiesPreviewNotificationsEnabled',
]


class GetOwnerNotificationSettingPropertiesNotificationsToSend(str, Enum):
    NONE = "none"
    FAILURE = "failure"
    ALL = "all"


class GetServiceNotificationOverridePropertiesNotificationsToSend(str, Enum):
    DEFAULT = "default"
    NONE = "none"
    FAILURE = "failure"
    ALL = "all"


class GetServiceNotificationOverridePropertiesPreviewNotificationsEnabled(str, Enum):
    DEFAULT = "default"
    FALSE = "false"
    TRUE = "true"


class NotificationOverrideWithCursorOverridePropertiesNotificationsToSend(str, Enum):
    DEFAULT = "default"
    NONE = "none"
    FAILURE = "failure"
    ALL = "all"


class NotificationOverrideWithCursorOverridePropertiesPreviewNotificationsEnabled(str, Enum):
    DEFAULT = "default"
    FALSE = "false"
    TRUE = "true"



# coding=utf-8
# *** WARNING: this file was generated by pulumigen. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from .. import _utilities
from ._enums import *

__all__ = [
    'RegistryCredential',
]

@pulumi.output_type
class RegistryCredential(dict):
    def __init__(__self__, *,
                 id: str,
                 name: str,
                 registry: 'RegistryCredentialRegistry',
                 username: str):
        """
        :param str id: Unique identifier for this credential
        :param str name: Descriptive name for this credential
        :param 'RegistryCredentialRegistry' registry: The registry to use this credential with
        :param str username: The username associated with the credential
        """
        pulumi.set(__self__, "id", id)
        pulumi.set(__self__, "name", name)
        pulumi.set(__self__, "registry", registry)
        pulumi.set(__self__, "username", username)

    @property
    @pulumi.getter
    def id(self) -> str:
        """
        Unique identifier for this credential
        """
        return pulumi.get(self, "id")

    @property
    @pulumi.getter
    def name(self) -> str:
        """
        Descriptive name for this credential
        """
        return pulumi.get(self, "name")

    @property
    @pulumi.getter
    def registry(self) -> 'RegistryCredentialRegistry':
        """
        The registry to use this credential with
        """
        return pulumi.get(self, "registry")

    @property
    @pulumi.getter
    def username(self) -> str:
        """
        The username associated with the credential
        """
        return pulumi.get(self, "username")





# coding=utf-8
# *** WARNING: this file was generated by pulumigen. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload, Awaitable
from .. import _utilities
from ._enums import *

__all__ = [
    'GetServiceNotificationOverrideProperties',
    'AwaitableGetServiceNotificationOverrideProperties',
    'get_service_notification_override',
    'get_service_notification_override_output',
]

@pulumi.output_type
class GetServiceNotificationOverrideProperties:
    def __init__(__self__, notifications_to_send=None, preview_notifications_enabled=None, service_id=None):
        if notifications_to_send and not isinstance(notifications_to_send, str):
            raise TypeError("Expected argument 'notifications_to_send' to be a str")
        pulumi.set(__self__, "notifications_to_send", notifications_to_send)
        if preview_notifications_enabled and not isinstance(preview_notifications_enabled, str):
            raise TypeError("Expected argument 'preview_notifications_enabled' to be a str")
        pulumi.set(__self__, "preview_notifications_enabled", preview_notifications_enabled)
        if service_id and not isinstance(service_id, str):
            raise TypeError("Expected argument 'service_id' to be a str")
        pulumi.set(__self__, "service_id", service_id)

    @property
    @pulumi.getter(name="notificationsToSend")
    def notifications_to_send(self) -> 'GetServiceNotificationOverridePropertiesNotificationsToSend':
        return pulumi.get(self, "notifications_to_send")

    @property
    @pulumi.getter(name="previewNotificationsEnabled")
    def preview_notifications_enabled(self) -> 'GetServiceNotificationOverridePropertiesPreviewNotificationsEnabled':
        return pulumi.get(self, "preview_notifications_enabled")

    @property
    @pulumi.getter(name="serviceId")
    def service_id(self) -> str:
        return pulumi.get(self, "service_id")


class AwaitableGetServiceNotificationOverrideProperties(GetServiceNotificationOverrideProperties):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetServiceNotificationOverrideProperties(
            notifications_to_send=self.notifications_to_send,
            preview_notifications_enabled=self.preview_notifications_enabled,
            service_id=self.service_id)


def get_service_notification_override(service_id: Optional[str] = None,
                                      opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetServiceNotificationOverrideProperties:
    """
    Use this data source to access information about an existing resource.

    :param str service_id: The ID of the service
    """
    __args__ = dict()
    __args__['serviceId'] = service_id
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('render:notification-settings:getServiceNotificationOverride', __args__, opts=opts, typ=GetServiceNotificationOverrideProperties).value

    return AwaitableGetServiceNotificationOverrideProperties(
        notifications_to_send=pulumi.get(__ret__, 'notifications_to_send'),
        preview_notifications_enabled=pulumi.get(__ret__, 'preview_notifications_enabled'),
        service_id=pulumi.get(__ret__, 'service_id'))


@_utilities.lift_output_func(get_service_notification_override)
def get_service_notification_override_output(service_id: Optional[pulumi.Input[str]] = None,
                                             opts: Optional[pulumi.InvokeOptions] = None) -> pulumi.Output[GetServiceNotificationOverrideProperties]:
    """
    Use this data source to access information about an existing resource.

    :param str service_id: The ID of the service
    """
    ...



# coding=utf-8
# *** WARNING: this file was generated by pulumigen. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from .. import _utilities
from . import outputs
from ._enums import *

__all__ = [
    'CidrBlockAndDescription',
    'ListRedisItemProperties',
    'MaintenanceProperties',
    'Owner',
    'Redis',
    'RedisDetailMaintenanceProperties',
    'RedisOptions',
]

@pulumi.output_type
class CidrBlockAndDescription(dict):
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "cidrBlock":
            suggest = "cidr_block"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in CidrBlockAndDescription. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        CidrBlockAndDescription.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        CidrBlockAndDescription.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 cidr_block: str,
                 description: str):
        """
        :param str description: User-provided description of the CIDR block
        """
        pulumi.set(__self__, "cidr_block", cidr_block)
        pulumi.set(__self__, "description", description)

    @property
    @pulumi.getter(name="cidrBlock")
    def cidr_block(self) -> str:
        return pulumi.get(self, "cidr_block")

    @property
    @pulumi.getter
    def description(self) -> str:
        """
        User-provided description of the CIDR block
        """
        return pulumi.get(self, "description")


@pulumi.output_type
class ListRedisItemProperties(dict):
    def __init__(__self__, *,
                 cursor: Optional[str] = None,
                 redis: Optional['outputs.Redis'] = None):
        """
        :param 'Redis' redis: A Redis instance
        """
        if cursor is not None:
            pulumi.set(__self__, "cursor", cursor)
        if redis is not None:
            pulumi.set(__self__, "redis", redis)

    @property
    @pulumi.getter
    def cursor(self) -> Optional[str]:
        return pulumi.get(self, "cursor")

    @property
    @pulumi.getter
    def redis(self) -> Optional['outputs.Redis']:
        """
        A Redis instance
        """
        return pulumi.get(self, "redis")


@pulumi.output_type
class MaintenanceProperties(dict):
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "scheduledAt":
            suggest = "scheduled_at"
        elif key == "pendingMaintenanceBy":
            suggest = "pending_maintenance_by"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in MaintenanceProperties. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        MaintenanceProperties.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        MaintenanceProperties.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 id: str,
                 scheduled_at: str,
                 state: 'MaintenancePropertiesState',
                 type: str,
                 pending_maintenance_by: Optional[str] = None):
        """
        :param str pending_maintenance_by: If present, the maintenance run cannot be scheduled for later than this date-time.
        """
        pulumi.set(__self__, "id", id)
        pulumi.set(__self__, "scheduled_at", scheduled_at)
        pulumi.set(__self__, "state", state)
        pulumi.set(__self__, "type", type)
        if pending_maintenance_by is not None:
            pulumi.set(__self__, "pending_maintenance_by", pending_maintenance_by)

    @property
    @pulumi.getter
    def id(self) -> str:
        return pulumi.get(self, "id")

    @property
    @pulumi.getter(name="scheduledAt")
    def scheduled_at(self) -> str:
        return pulumi.get(self, "scheduled_at")

    @property
    @pulumi.getter
    def state(self) -> 'MaintenancePropertiesState':
        return pulumi.get(self, "state")

    @property
    @pulumi.getter
    def type(self) -> str:
        return pulumi.get(self, "type")

    @property
    @pulumi.getter(name="pendingMaintenanceBy")
    def pending_maintenance_by(self) -> Optional[str]:
        """
        If present, the maintenance run cannot be scheduled for later than this date-time.
        """
        return pulumi.get(self, "pending_maintenance_by")


@pulumi.output_type
class Owner(dict):
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "twoFactorAuthEnabled":
            suggest = "two_factor_auth_enabled"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in Owner. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        Owner.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        Owner.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 email: str,
                 id: str,
                 name: str,
                 type: 'OwnerType',
                 two_factor_auth_enabled: Optional[bool] = None):
        """
        :param bool two_factor_auth_enabled: Whether two-factor authentication is enabled for the owner. Only present for user owners.
        """
        pulumi.set(__self__, "email", email)
        pulumi.set(__self__, "id", id)
        pulumi.set(__self__, "name", name)
        pulumi.set(__self__, "type", type)
        if two_factor_auth_enabled is not None:
            pulumi.set(__self__, "two_factor_auth_enabled", two_factor_auth_enabled)

    @property
    @pulumi.getter
    def email(self) -> str:
        return pulumi.get(self, "email")

    @property
    @pulumi.getter
    def id(self) -> str:
        return pulumi.get(self, "id")

    @property
    @pulumi.getter
    def name(self) -> str:
        return pulumi.get(self, "name")

    @property
    @pulumi.getter
    def type(self) -> 'OwnerType':
        return pulumi.get(self, "type")

    @property
    @pulumi.getter(name="twoFactorAuthEnabled")
    def two_factor_auth_enabled(self) -> Optional[bool]:
        """
        Whether two-factor authentication is enabled for the owner. Only present for user owners.
        """
        return pulumi.get(self, "two_factor_auth_enabled")


@pulumi.output_type
class Redis(dict):
    """
    A Redis instance
    """
    def __init__(__self__, *,
                 created_at: str,
                 dashboard_url: str,
                 id: str,
                 ip_allow_list: Sequence['outputs.CidrBlockAndDescription'],
                 name: str,
                 options: 'outputs.RedisOptions',
                 owner: 'outputs.Owner',
                 plan: 'RedisPlan',
                 region: Optional['RedisRegion'] = None,
                 status: 'RedisStatus',
                 updated_at: str,
                 version: str,
                 environment_id: Optional[str] = None):
        """
        A Redis instance
        :param str created_at: The creation time of the Redis instance
        :param str dashboard_url: The URL to view the Redis instance in the Render Dashboard
        :param str id: The ID of the Redis instance
        :param Sequence['CidrBlockAndDescription'] ip_allow_list: The IP allow list for the Redis instance
        :param str name: The name of the Redis instance
        :param 'RedisOptions' options: Options for a Redis instance
        :param 'RedisRegion' region: Defaults to "oregon"
        :param str updated_at: The last updated time of the Redis instance
        :param str version: The version of Redis
        :param str environment_id: The ID of the environment the Redis instance is associated with
        """
        pulumi.set(__self__, "created_at", created_at)
        pulumi.set(__self__, "dashboard_url", dashboard_url)
        pulumi.set(__self__, "id", id)
        pulumi.set(__self__, "ip_allow_list", ip_allow_list)
        pulumi.set(__self__, "name", name)
        pulumi.set(__self__, "options", options)
        pulumi.set(__self__, "owner", owner)
        pulumi.set(__self__, "plan", plan)
        if region is None:
            region = 'oregon'
        pulumi.set(__self__, "region", region)
        pulumi.set(__self__, "status", status)
        pulumi.set(__self__, "updated_at", updated_at)
        pulumi.set(__self__, "version", version)
        if environment_id is not None:
            pulumi.set(__self__, "environment_id", environment_id)

    @property
    @pulumi.getter(name="createdAt")
    def created_at(self) -> str:
        """
        The creation time of the Redis instance
        """
        return pulumi.get(self, "created_at")

    @property
    @pulumi.getter(name="dashboardUrl")
    def dashboard_url(self) -> str:
        """
        The URL to view the Redis instance in the Render Dashboard
        """
        return pulumi.get(self, "dashboard_url")

    @property
    @pulumi.getter
    def id(self) -> str:
        """
        The ID of the Redis instance
        """
        return pulumi.get(self, "id")

    @property
    @pulumi.getter(name="ipAllowList")
    def ip_allow_list(self) -> Sequence['outputs.CidrBlockAndDescription']:
        """
        The IP allow list for the Redis instance
        """
        return pulumi.get(self, "ip_allow_list")

    @property
    @pulumi.getter
    def name(self) -> str:
        """
        The name of the Redis instance
        """
        return pulumi.get(self, "name")

    @property
    @pulumi.getter
    def options(self) -> 'outputs.RedisOptions':
        """
        Options for a Redis instance
        """
        return pulumi.get(self, "options")

    @property
    @pulumi.getter
    def owner(self) -> 'outputs.Owner':
        return pulumi.get(self, "owner")

    @property
    @pulumi.getter
    def plan(self) -> 'RedisPlan':
        return pulumi.get(self, "plan")

    @property
    @pulumi.getter
    def region(self) -> 'RedisRegion':
        """
        Defaults to "oregon"
        """
        return pulumi.get(self, "region")

    @property
    @pulumi.getter
    def status(self) -> 'RedisStatus':
        return pulumi.get(self, "status")

    @property
    @pulumi.getter(name="updatedAt")
    def updated_at(self) -> str:
        """
        The last updated time of the Redis instance
        """
        return pulumi.get(self, "updated_at")

    @property
    @pulumi.getter
    def version(self) -> str:
        """
        The version of Redis
        """
        return pulumi.get(self, "version")

    @property
    @pulumi.getter(name="environmentId")
    def environment_id(self) -> Optional[str]:
        """
        The ID of the environment the Redis instance is associated with
        """
        return pulumi.get(self, "environment_id")


@pulumi.output_type
class RedisDetailMaintenanceProperties(dict):
    def __init__(__self__, *,
                 id: str,
                 scheduled_at: str,
                 state: 'RedisDetailMaintenancePropertiesState',
                 type: str,
                 pending_maintenance_by: Optional[str] = None):
        """
        :param str pending_maintenance_by: If present, the maintenance run cannot be scheduled for later than this date-time.
        """
        pulumi.set(__self__, "id", id)
        pulumi.set(__self__, "scheduled_at", scheduled_at)
        pulumi.set(__self__, "state", state)
        pulumi.set(__self__, "type", type)
        if pending_maintenance_by is not None:
            pulumi.set(__self__, "pending_maintenance_by", pending_maintenance_by)

    @property
    @pulumi.getter
    def id(self) -> str:
        return pulumi.get(self, "id")

    @property
    @pulumi.getter(name="scheduledAt")
    def scheduled_at(self) -> str:
        return pulumi.get(self, "scheduled_at")

    @property
    @pulumi.getter
    def state(self) -> 'RedisDetailMaintenancePropertiesState':
        return pulumi.get(self, "state")

    @property
    @pulumi.getter
    def type(self) -> str:
        return pulumi.get(self, "type")

    @property
    @pulumi.getter(name="pendingMaintenanceBy")
    def pending_maintenance_by(self) -> Optional[str]:
        """
        If present, the maintenance run cannot be scheduled for later than this date-time.
        """
        return pulumi.get(self, "pending_maintenance_by")


@pulumi.output_type
class RedisOptions(dict):
    """
    Options for a Redis instance
    """
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "maxmemoryPolicy":
            suggest = "maxmemory_policy"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in RedisOptions. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        RedisOptions.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        RedisOptions.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 maxmemory_policy: Optional[str] = None):
        """
        Options for a Redis instance
        """
        if maxmemory_policy is not None:
            pulumi.set(__self__, "maxmemory_policy", maxmemory_policy)

    @property
    @pulumi.getter(name="maxmemoryPolicy")
    def maxmemory_policy(self) -> Optional[str]:
        return pulumi.get(self, "maxmemory_policy")





# coding=utf-8
# *** WARNING: this file was generated by pulumigen. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

from enum import Enum

__all__ = [
    'BlueprintWithCursorBlueprintPropertiesStatus',
    'GetBlueprintPropertiesResourcesItemPropertiesType',
    'GetBlueprintPropertiesStatus',
    'SyncWithCursorSyncPropertiesState',
]


class BlueprintWithCursorBlueprintPropertiesStatus(str, Enum):
    CREATED = "created"
    PAUSED = "paused"
    IN_SYNC = "in_sync"
    SYNCING = "syncing"
    ERROR = "error"


class GetBlueprintPropertiesResourcesItemPropertiesType(str, Enum):
    """
    type of the resource (ex. web_service or postgres)
    """
    STATIC_SITE = "static_site"
    WEB_SERVICE = "web_service"
    PRIVATE_SERVICE = "private_service"
    BACKGROUND_WORKER = "background_worker"
    CRON_JOB = "cron_job"
    REDIS = "redis"
    POSTGRES = "postgres"
    ENVIRONMENT_GROUP = "environment_group"


class GetBlueprintPropertiesStatus(str, Enum):
    CREATED = "created"
    PAUSED = "paused"
    IN_SYNC = "in_sync"
    SYNCING = "syncing"
    ERROR = "error"


class SyncWithCursorSyncPropertiesState(str, Enum):
    CREATED = "created"
    PENDING = "pending"
    RUNNING = "running"
    ERROR = "error"
    SUCCESS = "success"



# coding=utf-8
# *** WARNING: this file was generated by pulumigen. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload, Awaitable
from .. import _utilities
from . import outputs
from ._enums import *

__all__ = [
    'GetCronJob',
    'AwaitableGetCronJob',
    'get_cron_job',
    'get_cron_job_output',
]

@pulumi.output_type
class GetCronJob:
    def __init__(__self__, auto_deploy=None, branch=None, build_filter=None, created_at=None, dashboard_url=None, environment_id=None, id=None, image_path=None, name=None, notify_on_fail=None, owner_id=None, registry_credential=None, repo=None, root_dir=None, service_details=None, slug=None, suspended=None, suspenders=None, type=None, updated_at=None):
        if auto_deploy and not isinstance(auto_deploy, str):
            raise TypeError("Expected argument 'auto_deploy' to be a str")
        pulumi.set(__self__, "auto_deploy", auto_deploy)
        if branch and not isinstance(branch, str):
            raise TypeError("Expected argument 'branch' to be a str")
        pulumi.set(__self__, "branch", branch)
        if build_filter and not isinstance(build_filter, dict):
            raise TypeError("Expected argument 'build_filter' to be a dict")
        pulumi.set(__self__, "build_filter", build_filter)
        if created_at and not isinstance(created_at, str):
            raise TypeError("Expected argument 'created_at' to be a str")
        pulumi.set(__self__, "created_at", created_at)
        if dashboard_url and not isinstance(dashboard_url, str):
            raise TypeError("Expected argument 'dashboard_url' to be a str")
        pulumi.set(__self__, "dashboard_url", dashboard_url)
        if environment_id and not isinstance(environment_id, str):
            raise TypeError("Expected argument 'environment_id' to be a str")
        pulumi.set(__self__, "environment_id", environment_id)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if image_path and not isinstance(image_path, str):
            raise TypeError("Expected argument 'image_path' to be a str")
        pulumi.set(__self__, "image_path", image_path)
        if name and not isinstance(name, str):
            raise TypeError("Expected argument 'name' to be a str")
        pulumi.set(__self__, "name", name)
        if notify_on_fail and not isinstance(notify_on_fail, str):
            raise TypeError("Expected argument 'notify_on_fail' to be a str")
        pulumi.set(__self__, "notify_on_fail", notify_on_fail)
        if owner_id and not isinstance(owner_id, str):
            raise TypeError("Expected argument 'owner_id' to be a str")
        pulumi.set(__self__, "owner_id", owner_id)
        if registry_credential and not isinstance(registry_credential, dict):
            raise TypeError("Expected argument 'registry_credential' to be a dict")
        pulumi.set(__self__, "registry_credential", registry_credential)
        if repo and not isinstance(repo, str):
            raise TypeError("Expected argument 'repo' to be a str")
        pulumi.set(__self__, "repo", repo)
        if root_dir and not isinstance(root_dir, str):
            raise TypeError("Expected argument 'root_dir' to be a str")
        pulumi.set(__self__, "root_dir", root_dir)
        if service_details and not isinstance(service_details, dict):
            raise TypeError("Expected argument 'service_details' to be a dict")
        pulumi.set(__self__, "service_details", service_details)
        if slug and not isinstance(slug, str):
            raise TypeError("Expected argument 'slug' to be a str")
        pulumi.set(__self__, "slug", slug)
        if suspended and not isinstance(suspended, str):
            raise TypeError("Expected argument 'suspended' to be a str")
        pulumi.set(__self__, "suspended", suspended)
        if suspenders and not isinstance(suspenders, list):
            raise TypeError("Expected argument 'suspenders' to be a list")
        pulumi.set(__self__, "suspenders", suspenders)
        if type and not isinstance(type, str):
            raise TypeError("Expected argument 'type' to be a str")
        pulumi.set(__self__, "type", type)
        if updated_at and not isinstance(updated_at, str):
            raise TypeError("Expected argument 'updated_at' to be a str")
        pulumi.set(__self__, "updated_at", updated_at)

    @property
    @pulumi.getter(name="autoDeploy")
    def auto_deploy(self) -> 'ServiceAutoDeploy':
        return pulumi.get(self, "auto_deploy")

    @property
    @pulumi.getter
    def branch(self) -> Optional[str]:
        return pulumi.get(self, "branch")

    @property
    @pulumi.getter(name="buildFilter")
    def build_filter(self) -> Optional['outputs.BuildFilter']:
        return pulumi.get(self, "build_filter")

    @property
    @pulumi.getter(name="createdAt")
    def created_at(self) -> str:
        return pulumi.get(self, "created_at")

    @property
    @pulumi.getter(name="dashboardUrl")
    def dashboard_url(self) -> str:
        """
        The URL to view the service in the Render Dashboard
        """
        return pulumi.get(self, "dashboard_url")

    @property
    @pulumi.getter(name="environmentId")
    def environment_id(self) -> Optional[str]:
        return pulumi.get(self, "environment_id")

    @property
    @pulumi.getter
    def id(self) -> str:
        return pulumi.get(self, "id")

    @property
    @pulumi.getter(name="imagePath")
    def image_path(self) -> Optional[str]:
        return pulumi.get(self, "image_path")

    @property
    @pulumi.getter
    def name(self) -> str:
        return pulumi.get(self, "name")

    @property
    @pulumi.getter(name="notifyOnFail")
    def notify_on_fail(self) -> 'ServiceNotifyOnFail':
        return pulumi.get(self, "notify_on_fail")

    @property
    @pulumi.getter(name="ownerId")
    def owner_id(self) -> str:
        return pulumi.get(self, "owner_id")

    @property
    @pulumi.getter(name="registryCredential")
    def registry_credential(self) -> Optional['outputs.RegistryCredentialSummary']:
        return pulumi.get(self, "registry_credential")

    @property
    @pulumi.getter
    def repo(self) -> Optional[str]:
        return pulumi.get(self, "repo")

    @property
    @pulumi.getter(name="rootDir")
    def root_dir(self) -> str:
        return pulumi.get(self, "root_dir")

    @property
    @pulumi.getter(name="serviceDetails")
    def service_details(self) -> Optional['outputs.CronJobDetailsOutput']:
        return pulumi.get(self, "service_details")

    @property
    @pulumi.getter
    def slug(self) -> str:
        return pulumi.get(self, "slug")

    @property
    @pulumi.getter
    def suspended(self) -> 'ServiceSuspended':
        return pulumi.get(self, "suspended")

    @property
    @pulumi.getter
    def suspenders(self) -> Sequence['ServiceSuspendersItem']:
        return pulumi.get(self, "suspenders")

    @property
    @pulumi.getter
    def type(self) -> Optional[str]:
        return pulumi.get(self, "type")

    @property
    @pulumi.getter(name="updatedAt")
    def updated_at(self) -> str:
        return pulumi.get(self, "updated_at")


class AwaitableGetCronJob(GetCronJob):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetCronJob(
            auto_deploy=self.auto_deploy,
            branch=self.branch,
            build_filter=self.build_filter,
            created_at=self.created_at,
            dashboard_url=self.dashboard_url,
            environment_id=self.environment_id,
            id=self.id,
            image_path=self.image_path,
            name=self.name,
            notify_on_fail=self.notify_on_fail,
            owner_id=self.owner_id,
            registry_credential=self.registry_credential,
            repo=self.repo,
            root_dir=self.root_dir,
            service_details=self.service_details,
            slug=self.slug,
            suspended=self.suspended,
            suspenders=self.suspenders,
            type=self.type,
            updated_at=self.updated_at)


def get_cron_job(service_id: Optional[str] = None,
                 opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetCronJob:
    """
    Use this data source to access information about an existing resource.

    :param str service_id: The ID of the service
    """
    __args__ = dict()
    __args__['serviceId'] = service_id
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('render:services:getCronJob', __args__, opts=opts, typ=GetCronJob).value

    return AwaitableGetCronJob(
        auto_deploy=pulumi.get(__ret__, 'auto_deploy'),
        branch=pulumi.get(__ret__, 'branch'),
        build_filter=pulumi.get(__ret__, 'build_filter'),
        created_at=pulumi.get(__ret__, 'created_at'),
        dashboard_url=pulumi.get(__ret__, 'dashboard_url'),
        environment_id=pulumi.get(__ret__, 'environment_id'),
        id=pulumi.get(__ret__, 'id'),
        image_path=pulumi.get(__ret__, 'image_path'),
        name=pulumi.get(__ret__, 'name'),
        notify_on_fail=pulumi.get(__ret__, 'notify_on_fail'),
        owner_id=pulumi.get(__ret__, 'owner_id'),
        registry_credential=pulumi.get(__ret__, 'registry_credential'),
        repo=pulumi.get(__ret__, 'repo'),
        root_dir=pulumi.get(__ret__, 'root_dir'),
        service_details=pulumi.get(__ret__, 'service_details'),
        slug=pulumi.get(__ret__, 'slug'),
        suspended=pulumi.get(__ret__, 'suspended'),
        suspenders=pulumi.get(__ret__, 'suspenders'),
        type=pulumi.get(__ret__, 'type'),
        updated_at=pulumi.get(__ret__, 'updated_at'))


@_utilities.lift_output_func(get_cron_job)
def get_cron_job_output(service_id: Optional[pulumi.Input[str]] = None,
                        opts: Optional[pulumi.InvokeOptions] = None) -> pulumi.Output[GetCronJob]:
    """
    Use this data source to access information about an existing resource.

    :param str service_id: The ID of the service
    """
    ...



# coding=utf-8
# *** WARNING: this file was generated by pulumigen. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from .. import _utilities
from . import outputs
from ._enums import *

__all__ = ['PreviewServiceArgs', 'PreviewService']

@pulumi.input_type
class PreviewServiceArgs:
    def __init__(__self__, *,
                 image_path: pulumi.Input[str],
                 name: Optional[pulumi.Input[str]] = None,
                 plan: Optional[pulumi.Input['Plan']] = None,
                 service_id: Optional[pulumi.Input[str]] = None):
        """
        The set of arguments for constructing a PreviewService resource.
        :param pulumi.Input[str] image_path: Must be either a full URL or the relative path to an image. If a relative path, Render uses the base service's image URL as its root. For example, if the base service's image URL is `docker.io/library/nginx:latest`, then valid values are: `docker.io/library/nginx:<any tag or SHA>`, `library/nginx:<any tag or SHA>`, or `nginx:<any tag or SHA>`. Note that the path must match (only the tag or SHA can vary).
        :param pulumi.Input[str] name: A name for the service preview instance. If not specified, Render generates the name using the base service's name and the specified tag or SHA.
        :param pulumi.Input['Plan'] plan: The instance type to use for the preview instance. Note that base services with any paid instance type can't create preview instances with the `free` instance type.
        :param pulumi.Input[str] service_id: The ID of the service
        """
        pulumi.set(__self__, "image_path", image_path)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if plan is not None:
            pulumi.set(__self__, "plan", plan)
        if service_id is not None:
            pulumi.set(__self__, "service_id", service_id)

    @property
    @pulumi.getter(name="imagePath")
    def image_path(self) -> pulumi.Input[str]:
        """
        Must be either a full URL or the relative path to an image. If a relative path, Render uses the base service's image URL as its root. For example, if the base service's image URL is `docker.io/library/nginx:latest`, then valid values are: `docker.io/library/nginx:<any tag or SHA>`, `library/nginx:<any tag or SHA>`, or `nginx:<any tag or SHA>`. Note that the path must match (only the tag or SHA can vary).
        """
        return pulumi.get(self, "image_path")

    @image_path.setter
    def image_path(self, value: pulumi.Input[str]):
        pulumi.set(self, "image_path", value)

    @property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[str]]:
        """
        A name for the service preview instance. If not specified, Render generates the name using the base service's name and the specified tag or SHA.
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "name", value)

    @property
    @pulumi.getter
    def plan(self) -> Optional[pulumi.Input['Plan']]:
        """
        The instance type to use for the preview instance. Note that base services with any paid instance type can't create preview instances with the `free` instance type.
        """
        return pulumi.get(self, "plan")

    @plan.setter
    def plan(self, value: Optional[pulumi.Input['Plan']]):
        pulumi.set(self, "plan", value)

    @property
    @pulumi.getter(name="serviceId")
    def service_id(self) -> Optional[pulumi.Input[str]]:
        """
        The ID of the service
        """
        return pulumi.get(self, "service_id")

    @service_id.setter
    def service_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "service_id", value)


class PreviewService(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 image_path: Optional[pulumi.Input[str]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 plan: Optional[pulumi.Input['Plan']] = None,
                 service_id: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        """
        Create a PreviewService resource with the given unique name, props, and options.
        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] image_path: Must be either a full URL or the relative path to an image. If a relative path, Render uses the base service's image URL as its root. For example, if the base service's image URL is `docker.io/library/nginx:latest`, then valid values are: `docker.io/library/nginx:<any tag or SHA>`, `library/nginx:<any tag or SHA>`, or `nginx:<any tag or SHA>`. Note that the path must match (only the tag or SHA can vary).
        :param pulumi.Input[str] name: A name for the service preview instance. If not specified, Render generates the name using the base service's name and the specified tag or SHA.
        :param pulumi.Input['Plan'] plan: The instance type to use for the preview instance. Note that base services with any paid instance type can't create preview instances with the `free` instance type.
        :param pulumi.Input[str] service_id: The ID of the service
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: PreviewServiceArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        Create a PreviewService resource with the given unique name, props, and options.
        :param str resource_name: The name of the resource.
        :param PreviewServiceArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(PreviewServiceArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 image_path: Optional[pulumi.Input[str]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 plan: Optional[pulumi.Input['Plan']] = None,
                 service_id: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = PreviewServiceArgs.__new__(PreviewServiceArgs)

            if image_path is None and not opts.urn:
                raise TypeError("Missing required property 'image_path'")
            __props__.__dict__["image_path"] = image_path
            __props__.__dict__["name"] = name
            __props__.__dict__["plan"] = plan
            __props__.__dict__["service_id"] = service_id
            __props__.__dict__["deploy_id"] = None
            __props__.__dict__["service"] = None
        super(PreviewService, __self__).__init__(
            'render:services:PreviewService',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None) -> 'PreviewService':
        """
        Get an existing PreviewService resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = PreviewServiceArgs.__new__(PreviewServiceArgs)

        __props__.__dict__["deploy_id"] = None
        __props__.__dict__["image_path"] = None
        __props__.__dict__["name"] = None
        __props__.__dict__["plan"] = None
        __props__.__dict__["service"] = None
        return PreviewService(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter(name="deployId")
    def deploy_id(self) -> pulumi.Output[Optional[str]]:
        return pulumi.get(self, "deploy_id")

    @property
    @pulumi.getter(name="imagePath")
    def image_path(self) -> pulumi.Output[str]:
        """
        Must be either a full URL or the relative path to an image. If a relative path, Render uses the base service's image URL as its root. For example, if the base service's image URL is `docker.io/library/nginx:latest`, then valid values are: `docker.io/library/nginx:<any tag or SHA>`, `library/nginx:<any tag or SHA>`, or `nginx:<any tag or SHA>`. Note that the path must match (only the tag or SHA can vary).
        """
        return pulumi.get(self, "image_path")

    @property
    @pulumi.getter
    def name(self) -> pulumi.Output[Optional[str]]:
        """
        A name for the service preview instance. If not specified, Render generates the name using the base service's name and the specified tag or SHA.
        """
        return pulumi.get(self, "name")

    @property
    @pulumi.getter
    def plan(self) -> pulumi.Output[Optional['Plan']]:
        """
        The instance type to use for the preview instance. Note that base services with any paid instance type can't create preview instances with the `free` instance type.
        """
        return pulumi.get(self, "plan")

    @property
    @pulumi.getter
    def service(self) -> pulumi.Output[Optional['outputs.Service']]:
        return pulumi.get(self, "service")




# coding=utf-8
# *** WARNING: this file was generated by pulumigen. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

from enum import Enum

__all__ = [
    'ServiceLinkType',
]


class ServiceLinkType(str, Enum):
    STATIC = "static"
    WEB = "web"
    PSERV = "pserv"
    WORKER = "worker"
    CRON = "cron"



# coding=utf-8
# *** WARNING: this file was generated by pulumigen. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload, Awaitable
from .. import _utilities
from ._enums import *

__all__ = [
    'RegistryCredential',
    'AwaitableRegistryCredential',
    'get_registry_credential',
    'get_registry_credential_output',
]

@pulumi.output_type
class RegistryCredential:
    def __init__(__self__, id=None, name=None, registry=None, username=None):
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if name and not isinstance(name, str):
            raise TypeError("Expected argument 'name' to be a str")
        pulumi.set(__self__, "name", name)
        if registry and not isinstance(registry, str):
            raise TypeError("Expected argument 'registry' to be a str")
        pulumi.set(__self__, "registry", registry)
        if username and not isinstance(username, str):
            raise TypeError("Expected argument 'username' to be a str")
        pulumi.set(__self__, "username", username)

    @property
    @pulumi.getter
    def id(self) -> str:
        """
        Unique identifier for this credential
        """
        return pulumi.get(self, "id")

    @property
    @pulumi.getter
    def name(self) -> str:
        """
        Descriptive name for this credential
        """
        return pulumi.get(self, "name")

    @property
    @pulumi.getter
    def registry(self) -> 'RegistryCredentialRegistry':
        """
        The registry to use this credential with
        """
        return pulumi.get(self, "registry")

    @property
    @pulumi.getter
    def username(self) -> str:
        """
        The username associated with the credential
        """
        return pulumi.get(self, "username")


class AwaitableRegistryCredential(RegistryCredential):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return RegistryCredential(
            id=self.id,
            name=self.name,
            registry=self.registry,
            username=self.username)


def get_registry_credential(registry_credential_id: Optional[str] = None,
                            opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableRegistryCredential:
    """
    Use this data source to access information about an existing resource.

    :param str registry_credential_id: The ID of the registry credential
    """
    __args__ = dict()
    __args__['registryCredentialId'] = registry_credential_id
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('render:registrycredentials:getRegistryCredential', __args__, opts=opts, typ=RegistryCredential).value

    return AwaitableRegistryCredential(
        id=pulumi.get(__ret__, 'id'),
        name=pulumi.get(__ret__, 'name'),
        registry=pulumi.get(__ret__, 'registry'),
        username=pulumi.get(__ret__, 'username'))


@_utilities.lift_output_func(get_registry_credential)
def get_registry_credential_output(registry_credential_id: Optional[pulumi.Input[str]] = None,
                                   opts: Optional[pulumi.InvokeOptions] = None) -> pulumi.Output[RegistryCredential]:
    """
    Use this data source to access information about an existing resource.

    :param str registry_credential_id: The ID of the registry credential
    """
    ...



# coding=utf-8
# *** WARNING: this file was generated by pulumigen. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from .. import _utilities
from . import outputs
from ._enums import *

__all__ = [
    'EnvGroupMeta',
    'EnvVar',
    'SecretFile',
    'ServiceLink',
]

@pulumi.output_type
class EnvGroupMeta(dict):
    def __init__(__self__, *,
                 created_at: str,
                 id: str,
                 name: str,
                 owner_id: str,
                 service_links: Sequence['outputs.ServiceLink'],
                 updated_at: str,
                 environment_id: Optional[str] = None):
        """
        :param Sequence['ServiceLink'] service_links: List of serviceIds linked to the envGroup
        """
        pulumi.set(__self__, "created_at", created_at)
        pulumi.set(__self__, "id", id)
        pulumi.set(__self__, "name", name)
        pulumi.set(__self__, "owner_id", owner_id)
        pulumi.set(__self__, "service_links", service_links)
        pulumi.set(__self__, "updated_at", updated_at)
        if environment_id is not None:
            pulumi.set(__self__, "environment_id", environment_id)

    @property
    @pulumi.getter(name="createdAt")
    def created_at(self) -> str:
        return pulumi.get(self, "created_at")

    @property
    @pulumi.getter
    def id(self) -> str:
        return pulumi.get(self, "id")

    @property
    @pulumi.getter
    def name(self) -> str:
        return pulumi.get(self, "name")

    @property
    @pulumi.getter(name="ownerId")
    def owner_id(self) -> str:
        return pulumi.get(self, "owner_id")

    @property
    @pulumi.getter(name="serviceLinks")
    def service_links(self) -> Sequence['outputs.ServiceLink']:
        """
        List of serviceIds linked to the envGroup
        """
        return pulumi.get(self, "service_links")

    @property
    @pulumi.getter(name="updatedAt")
    def updated_at(self) -> str:
        return pulumi.get(self, "updated_at")

    @property
    @pulumi.getter(name="environmentId")
    def environment_id(self) -> Optional[str]:
        return pulumi.get(self, "environment_id")


@pulumi.output_type
class EnvVar(dict):
    def __init__(__self__, *,
                 key: str,
                 value: str):
        pulumi.set(__self__, "key", key)
        pulumi.set(__self__, "value", value)

    @property
    @pulumi.getter
    def key(self) -> str:
        return pulumi.get(self, "key")

    @property
    @pulumi.getter
    def value(self) -> str:
        return pulumi.get(self, "value")


@pulumi.output_type
class SecretFile(dict):
    def __init__(__self__, *,
                 content: str,
                 name: str):
        pulumi.set(__self__, "content", content)
        pulumi.set(__self__, "name", name)

    @property
    @pulumi.getter
    def content(self) -> str:
        return pulumi.get(self, "content")

    @property
    @pulumi.getter
    def name(self) -> str:
        return pulumi.get(self, "name")


@pulumi.output_type
class ServiceLink(dict):
    def __init__(__self__, *,
                 id: str,
                 name: str,
                 type: 'ServiceLinkType'):
        pulumi.set(__self__, "id", id)
        pulumi.set(__self__, "name", name)
        pulumi.set(__self__, "type", type)

    @property
    @pulumi.getter
    def id(self) -> str:
        return pulumi.get(self, "id")

    @property
    @pulumi.getter
    def name(self) -> str:
        return pulumi.get(self, "name")

    @property
    @pulumi.getter
    def type(self) -> 'ServiceLinkType':
        return pulumi.get(self, "type")





# coding=utf-8
# *** WARNING: this file was generated by pulumigen. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from .. import _utilities
from . import outputs
from ._enums import *
from ._inputs import *

__all__ = ['StaticSiteArgs', 'StaticSite']

@pulumi.input_type
class StaticSiteArgs:
    def __init__(__self__, *,
                 name: pulumi.Input[str],
                 owner_id: pulumi.Input[str],
                 auto_deploy: Optional[pulumi.Input['ServiceCreateAutoDeploy']] = None,
                 branch: Optional[pulumi.Input[str]] = None,
                 build_filter: Optional[pulumi.Input['BuildFilterArgs']] = None,
                 env_vars: Optional[pulumi.Input[Sequence[pulumi.Input['EnvVarInputArgs']]]] = None,
                 image: Optional[pulumi.Input['ImageArgs']] = None,
                 repo: Optional[pulumi.Input[str]] = None,
                 root_dir: Optional[pulumi.Input[str]] = None,
                 secret_files: Optional[pulumi.Input[Sequence[pulumi.Input['SecretFileInputArgs']]]] = None,
                 service_details: Optional[pulumi.Input['StaticSiteDetailsCreateArgs']] = None,
                 type: Optional[pulumi.Input[str]] = None):
        """
        The set of arguments for constructing a StaticSite resource.
        :param pulumi.Input[str] branch: If left empty, this will fall back to the default branch of the repository
        :param pulumi.Input[str] repo: Do not include the branch in the repo string. You can instead supply a 'branch' parameter.
        """
        pulumi.set(__self__, "name", name)
        pulumi.set(__self__, "owner_id", owner_id)
        if auto_deploy is None:
            auto_deploy = 'yes'
        if auto_deploy is not None:
            pulumi.set(__self__, "auto_deploy", auto_deploy)
        if branch is not None:
            pulumi.set(__self__, "branch", branch)
        if build_filter is not None:
            pulumi.set(__self__, "build_filter", build_filter)
        if env_vars is not None:
            pulumi.set(__self__, "env_vars", env_vars)
        if image is not None:
            pulumi.set(__self__, "image", image)
        if repo is not None:
            pulumi.set(__self__, "repo", repo)
        if root_dir is not None:
            pulumi.set(__self__, "root_dir", root_dir)
        if secret_files is not None:
            pulumi.set(__self__, "secret_files", secret_files)
        if service_details is not None:
            pulumi.set(__self__, "service_details", service_details)
        if type is None:
            type = 'static_site'
        if type is not None:
            pulumi.set(__self__, "type", type)

    @property
    @pulumi.getter
    def name(self) -> pulumi.Input[str]:
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: pulumi.Input[str]):
        pulumi.set(self, "name", value)

    @property
    @pulumi.getter(name="ownerId")
    def owner_id(self) -> pulumi.Input[str]:
        return pulumi.get(self, "owner_id")

    @owner_id.setter
    def owner_id(self, value: pulumi.Input[str]):
        pulumi.set(self, "owner_id", value)

    @property
    @pulumi.getter(name="autoDeploy")
    def auto_deploy(self) -> Optional[pulumi.Input['ServiceCreateAutoDeploy']]:
        return pulumi.get(self, "auto_deploy")

    @auto_deploy.setter
    def auto_deploy(self, value: Optional[pulumi.Input['ServiceCreateAutoDeploy']]):
        pulumi.set(self, "auto_deploy", value)

    @property
    @pulumi.getter
    def branch(self) -> Optional[pulumi.Input[str]]:
        """
        If left empty, this will fall back to the default branch of the repository
        """
        return pulumi.get(self, "branch")

    @branch.setter
    def branch(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "branch", value)

    @property
    @pulumi.getter(name="buildFilter")
    def build_filter(self) -> Optional[pulumi.Input['BuildFilterArgs']]:
        return pulumi.get(self, "build_filter")

    @build_filter.setter
    def build_filter(self, value: Optional[pulumi.Input['BuildFilterArgs']]):
        pulumi.set(self, "build_filter", value)

    @property
    @pulumi.getter(name="envVars")
    def env_vars(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['EnvVarInputArgs']]]]:
        return pulumi.get(self, "env_vars")

    @env_vars.setter
    def env_vars(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['EnvVarInputArgs']]]]):
        pulumi.set(self, "env_vars", value)

    @property
    @pulumi.getter
    def image(self) -> Optional[pulumi.Input['ImageArgs']]:
        return pulumi.get(self, "image")

    @image.setter
    def image(self, value: Optional[pulumi.Input['ImageArgs']]):
        pulumi.set(self, "image", value)

    @property
    @pulumi.getter
    def repo(self) -> Optional[pulumi.Input[str]]:
        """
        Do not include the branch in the repo string. You can instead supply a 'branch' parameter.
        """
        return pulumi.get(self, "repo")

    @repo.setter
    def repo(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "repo", value)

    @property
    @pulumi.getter(name="rootDir")
    def root_dir(self) -> Optional[pulumi.Input[str]]:
        return pulumi.get(self, "root_dir")

    @root_dir.setter
    def root_dir(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "root_dir", value)

    @property
    @pulumi.getter(name="secretFiles")
    def secret_files(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['SecretFileInputArgs']]]]:
        return pulumi.get(self, "secret_files")

    @secret_files.setter
    def secret_files(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['SecretFileInputArgs']]]]):
        pulumi.set(self, "secret_files", value)

    @property
    @pulumi.getter(name="serviceDetails")
    def service_details(self) -> Optional[pulumi.Input['StaticSiteDetailsCreateArgs']]:
        return pulumi.get(self, "service_details")

    @service_details.setter
    def service_details(self, value: Optional[pulumi.Input['StaticSiteDetailsCreateArgs']]):
        pulumi.set(self, "service_details", value)

    @property
    @pulumi.getter
    def type(self) -> Optional[pulumi.Input[str]]:
        return pulumi.get(self, "type")

    @type.setter
    def type(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "type", value)


class StaticSite(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 auto_deploy: Optional[pulumi.Input['ServiceCreateAutoDeploy']] = None,
                 branch: Optional[pulumi.Input[str]] = None,
                 build_filter: Optional[pulumi.Input[Union['BuildFilterArgs', 'BuildFilterArgsDict']]] = None,
                 env_vars: Optional[pulumi.Input[Sequence[pulumi.Input[Union['EnvVarInputArgs', 'EnvVarInputArgsDict']]]]] = None,
                 image: Optional[pulumi.Input[Union['ImageArgs', 'ImageArgsDict']]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 owner_id: Optional[pulumi.Input[str]] = None,
                 repo: Optional[pulumi.Input[str]] = None,
                 root_dir: Optional[pulumi.Input[str]] = None,
                 secret_files: Optional[pulumi.Input[Sequence[pulumi.Input[Union['SecretFileInputArgs', 'SecretFileInputArgsDict']]]]] = None,
                 service_details: Optional[pulumi.Input[Union['StaticSiteDetailsCreateArgs', 'StaticSiteDetailsCreateArgsDict']]] = None,
                 type: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        """
        ## Example Usage

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] branch: If left empty, this will fall back to the default branch of the repository
        :param pulumi.Input[str] repo: Do not include the branch in the repo string. You can instead supply a 'branch' parameter.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: StaticSiteArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        ## Example Usage

        :param str resource_name: The name of the resource.
        :param StaticSiteArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(StaticSiteArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 auto_deploy: Optional[pulumi.Input['ServiceCreateAutoDeploy']] = None,
                 branch: Optional[pulumi.Input[str]] = None,
                 build_filter: Optional[pulumi.Input[Union['BuildFilterArgs', 'BuildFilterArgsDict']]] = None,
                 env_vars: Optional[pulumi.Input[Sequence[pulumi.Input[Union['EnvVarInputArgs', 'EnvVarInputArgsDict']]]]] = None,
                 image: Optional[pulumi.Input[Union['ImageArgs', 'ImageArgsDict']]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 owner_id: Optional[pulumi.Input[str]] = None,
                 repo: Optional[pulumi.Input[str]] = None,
                 root_dir: Optional[pulumi.Input[str]] = None,
                 secret_files: Optional[pulumi.Input[Sequence[pulumi.Input[Union['SecretFileInputArgs', 'SecretFileInputArgsDict']]]]] = None,
                 service_details: Optional[pulumi.Input[Union['StaticSiteDetailsCreateArgs', 'StaticSiteDetailsCreateArgsDict']]] = None,
                 type: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = StaticSiteArgs.__new__(StaticSiteArgs)

            if auto_deploy is None:
                auto_deploy = 'yes'
            __props__.__dict__["auto_deploy"] = auto_deploy
            __props__.__dict__["branch"] = branch
            __props__.__dict__["build_filter"] = build_filter
            __props__.__dict__["env_vars"] = env_vars
            __props__.__dict__["image"] = image
            if name is None and not opts.urn:
                raise TypeError("Missing required property 'name'")
            __props__.__dict__["name"] = name
            if owner_id is None and not opts.urn:
                raise TypeError("Missing required property 'owner_id'")
            __props__.__dict__["owner_id"] = owner_id
            __props__.__dict__["repo"] = repo
            __props__.__dict__["root_dir"] = root_dir
            __props__.__dict__["secret_files"] = secret_files
            __props__.__dict__["service_details"] = service_details
            if type is None:
                type = 'static_site'
            __props__.__dict__["type"] = type
            __props__.__dict__["created_at"] = None
            __props__.__dict__["dashboard_url"] = None
            __props__.__dict__["environment_id"] = None
            __props__.__dict__["image_path"] = None
            __props__.__dict__["notify_on_fail"] = None
            __props__.__dict__["registry_credential"] = None
            __props__.__dict__["slug"] = None
            __props__.__dict__["suspended"] = None
            __props__.__dict__["suspenders"] = None
            __props__.__dict__["updated_at"] = None
        super(StaticSite, __self__).__init__(
            'render:services:StaticSite',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None) -> 'StaticSite':
        """
        Get an existing StaticSite resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = StaticSiteArgs.__new__(StaticSiteArgs)

        __props__.__dict__["auto_deploy"] = None
        __props__.__dict__["branch"] = None
        __props__.__dict__["build_filter"] = None
        __props__.__dict__["created_at"] = None
        __props__.__dict__["dashboard_url"] = None
        __props__.__dict__["env_vars"] = None
        __props__.__dict__["environment_id"] = None
        __props__.__dict__["image"] = None
        __props__.__dict__["image_path"] = None
        __props__.__dict__["name"] = None
        __props__.__dict__["notify_on_fail"] = None
        __props__.__dict__["owner_id"] = None
        __props__.__dict__["registry_credential"] = None
        __props__.__dict__["repo"] = None
        __props__.__dict__["root_dir"] = None
        __props__.__dict__["secret_files"] = None
        __props__.__dict__["service_details"] = None
        __props__.__dict__["slug"] = None
        __props__.__dict__["suspended"] = None
        __props__.__dict__["suspenders"] = None
        __props__.__dict__["type"] = None
        __props__.__dict__["updated_at"] = None
        return StaticSite(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter(name="autoDeploy")
    def auto_deploy(self) -> pulumi.Output[Optional['ServiceAutoDeploy']]:
        return pulumi.get(self, "auto_deploy")

    @property
    @pulumi.getter
    def branch(self) -> pulumi.Output[Optional[str]]:
        return pulumi.get(self, "branch")

    @property
    @pulumi.getter(name="buildFilter")
    def build_filter(self) -> pulumi.Output[Optional['outputs.BuildFilter']]:
        return pulumi.get(self, "build_filter")

    @property
    @pulumi.getter(name="createdAt")
    def created_at(self) -> pulumi.Output[Optional[str]]:
        return pulumi.get(self, "created_at")

    @property
    @pulumi.getter(name="dashboardUrl")
    def dashboard_url(self) -> pulumi.Output[Optional[str]]:
        """
        The URL to view the service in the Render Dashboard
        """
        return pulumi.get(self, "dashboard_url")

    @property
    @pulumi.getter(name="envVars")
    def env_vars(self) -> pulumi.Output[Optional[Sequence['outputs.EnvVarInput']]]:
        return pulumi.get(self, "env_vars")

    @property
    @pulumi.getter(name="environmentId")
    def environment_id(self) -> pulumi.Output[Optional[str]]:
        return pulumi.get(self, "environment_id")

    @property
    @pulumi.getter
    def image(self) -> pulumi.Output[Optional['outputs.Image']]:
        return pulumi.get(self, "image")

    @property
    @pulumi.getter(name="imagePath")
    def image_path(self) -> pulumi.Output[Optional[str]]:
        return pulumi.get(self, "image_path")

    @property
    @pulumi.getter
    def name(self) -> pulumi.Output[Optional[str]]:
        return pulumi.get(self, "name")

    @property
    @pulumi.getter(name="notifyOnFail")
    def notify_on_fail(self) -> pulumi.Output[Optional['ServiceNotifyOnFail']]:
        return pulumi.get(self, "notify_on_fail")

    @property
    @pulumi.getter(name="ownerId")
    def owner_id(self) -> pulumi.Output[Optional[str]]:
        return pulumi.get(self, "owner_id")

    @property
    @pulumi.getter(name="registryCredential")
    def registry_credential(self) -> pulumi.Output[Optional['outputs.RegistryCredentialSummary']]:
        return pulumi.get(self, "registry_credential")

    @property
    @pulumi.getter
    def repo(self) -> pulumi.Output[Optional[str]]:
        return pulumi.get(self, "repo")

    @property
    @pulumi.getter(name="rootDir")
    def root_dir(self) -> pulumi.Output[Optional[str]]:
        return pulumi.get(self, "root_dir")

    @property
    @pulumi.getter(name="secretFiles")
    def secret_files(self) -> pulumi.Output[Optional[Sequence['outputs.SecretFileInput']]]:
        return pulumi.get(self, "secret_files")

    @property
    @pulumi.getter(name="serviceDetails")
    def service_details(self) -> pulumi.Output[Optional['outputs.StaticSiteDetailsOutput']]:
        return pulumi.get(self, "service_details")

    @property
    @pulumi.getter
    def slug(self) -> pulumi.Output[Optional[str]]:
        return pulumi.get(self, "slug")

    @property
    @pulumi.getter
    def suspended(self) -> pulumi.Output[Optional['ServiceSuspended']]:
        return pulumi.get(self, "suspended")

    @property
    @pulumi.getter
    def suspenders(self) -> pulumi.Output[Optional[Sequence['ServiceSuspendersItem']]]:
        return pulumi.get(self, "suspenders")

    @property
    @pulumi.getter
    def type(self) -> pulumi.Output[Optional[str]]:
        return pulumi.get(self, "type")

    @property
    @pulumi.getter(name="updatedAt")
    def updated_at(self) -> pulumi.Output[Optional[str]]:
        return pulumi.get(self, "updated_at")




# coding=utf-8
# *** WARNING: this file was generated by pulumigen. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from .. import _utilities
from . import outputs
from ._enums import *

__all__ = [
    'Owner',
    'Project',
    'ProjectCreateEnvironmentInput',
    'ProjectWithCursor',
]

@pulumi.output_type
class Owner(dict):
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "twoFactorAuthEnabled":
            suggest = "two_factor_auth_enabled"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in Owner. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        Owner.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        Owner.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 email: str,
                 id: str,
                 name: str,
                 type: 'OwnerType',
                 two_factor_auth_enabled: Optional[bool] = None):
        """
        :param bool two_factor_auth_enabled: Whether two-factor authentication is enabled for the owner. Only present for user owners.
        """
        pulumi.set(__self__, "email", email)
        pulumi.set(__self__, "id", id)
        pulumi.set(__self__, "name", name)
        pulumi.set(__self__, "type", type)
        if two_factor_auth_enabled is not None:
            pulumi.set(__self__, "two_factor_auth_enabled", two_factor_auth_enabled)

    @property
    @pulumi.getter
    def email(self) -> str:
        return pulumi.get(self, "email")

    @property
    @pulumi.getter
    def id(self) -> str:
        return pulumi.get(self, "id")

    @property
    @pulumi.getter
    def name(self) -> str:
        return pulumi.get(self, "name")

    @property
    @pulumi.getter
    def type(self) -> 'OwnerType':
        return pulumi.get(self, "type")

    @property
    @pulumi.getter(name="twoFactorAuthEnabled")
    def two_factor_auth_enabled(self) -> Optional[bool]:
        """
        Whether two-factor authentication is enabled for the owner. Only present for user owners.
        """
        return pulumi.get(self, "two_factor_auth_enabled")


@pulumi.output_type
class Project(dict):
    """
    A project is a collection of environments
    """
    def __init__(__self__, *,
                 created_at: str,
                 environment_ids: Sequence[str],
                 id: str,
                 name: str,
                 owner: 'outputs.Owner',
                 updated_at: str):
        """
        A project is a collection of environments
        :param Sequence[str] environment_ids: The environments associated with the project
        :param str id: The ID of the project
        :param str name: The name of the project
        """
        pulumi.set(__self__, "created_at", created_at)
        pulumi.set(__self__, "environment_ids", environment_ids)
        pulumi.set(__self__, "id", id)
        pulumi.set(__self__, "name", name)
        pulumi.set(__self__, "owner", owner)
        pulumi.set(__self__, "updated_at", updated_at)

    @property
    @pulumi.getter(name="createdAt")
    def created_at(self) -> str:
        return pulumi.get(self, "created_at")

    @property
    @pulumi.getter(name="environmentIds")
    def environment_ids(self) -> Sequence[str]:
        """
        The environments associated with the project
        """
        return pulumi.get(self, "environment_ids")

    @property
    @pulumi.getter
    def id(self) -> str:
        """
        The ID of the project
        """
        return pulumi.get(self, "id")

    @property
    @pulumi.getter
    def name(self) -> str:
        """
        The name of the project
        """
        return pulumi.get(self, "name")

    @property
    @pulumi.getter
    def owner(self) -> 'outputs.Owner':
        return pulumi.get(self, "owner")

    @property
    @pulumi.getter(name="updatedAt")
    def updated_at(self) -> str:
        return pulumi.get(self, "updated_at")


@pulumi.output_type
class ProjectCreateEnvironmentInput(dict):
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "protectedStatus":
            suggest = "protected_status"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in ProjectCreateEnvironmentInput. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        ProjectCreateEnvironmentInput.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        ProjectCreateEnvironmentInput.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 name: str,
                 protected_status: Optional['ProjectCreateEnvironmentInputProtectedStatus'] = None):
        """
        :param 'ProjectCreateEnvironmentInputProtectedStatus' protected_status: Indicates whether an environment is `unprotected` or `protected`. Only admin users can perform destructive actions in `protected` environments.
        """
        pulumi.set(__self__, "name", name)
        if protected_status is not None:
            pulumi.set(__self__, "protected_status", protected_status)

    @property
    @pulumi.getter
    def name(self) -> str:
        return pulumi.get(self, "name")

    @property
    @pulumi.getter(name="protectedStatus")
    def protected_status(self) -> Optional['ProjectCreateEnvironmentInputProtectedStatus']:
        """
        Indicates whether an environment is `unprotected` or `protected`. Only admin users can perform destructive actions in `protected` environments.
        """
        return pulumi.get(self, "protected_status")


@pulumi.output_type
class ProjectWithCursor(dict):
    def __init__(__self__, *,
                 cursor: str,
                 project: 'outputs.Project'):
        """
        :param 'Project' project: A project is a collection of environments
        """
        pulumi.set(__self__, "cursor", cursor)
        pulumi.set(__self__, "project", project)

    @property
    @pulumi.getter
    def cursor(self) -> str:
        return pulumi.get(self, "cursor")

    @property
    @pulumi.getter
    def project(self) -> 'outputs.Project':
        """
        A project is a collection of environments
        """
        return pulumi.get(self, "project")





# coding=utf-8
# *** WARNING: this file was generated by pulumigen. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from .. import _utilities
from . import outputs
from ._enums import *

__all__ = ['RollbackDeployArgs', 'RollbackDeploy']

@pulumi.input_type
class RollbackDeployArgs:
    def __init__(__self__, *,
                 deploy_id: pulumi.Input[str],
                 service_id: Optional[pulumi.Input[str]] = None):
        """
        The set of arguments for constructing a RollbackDeploy resource.
        :param pulumi.Input[str] deploy_id: The ID of the deploy to roll back to
        :param pulumi.Input[str] service_id: The ID of the service
        """
        pulumi.set(__self__, "deploy_id", deploy_id)
        if service_id is not None:
            pulumi.set(__self__, "service_id", service_id)

    @property
    @pulumi.getter(name="deployId")
    def deploy_id(self) -> pulumi.Input[str]:
        """
        The ID of the deploy to roll back to
        """
        return pulumi.get(self, "deploy_id")

    @deploy_id.setter
    def deploy_id(self, value: pulumi.Input[str]):
        pulumi.set(self, "deploy_id", value)

    @property
    @pulumi.getter(name="serviceId")
    def service_id(self) -> Optional[pulumi.Input[str]]:
        """
        The ID of the service
        """
        return pulumi.get(self, "service_id")

    @service_id.setter
    def service_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "service_id", value)


class RollbackDeploy(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 deploy_id: Optional[pulumi.Input[str]] = None,
                 service_id: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        """
        Create a RollbackDeploy resource with the given unique name, props, and options.
        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] deploy_id: The ID of the deploy to roll back to
        :param pulumi.Input[str] service_id: The ID of the service
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: RollbackDeployArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        Create a RollbackDeploy resource with the given unique name, props, and options.
        :param str resource_name: The name of the resource.
        :param RollbackDeployArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(RollbackDeployArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 deploy_id: Optional[pulumi.Input[str]] = None,
                 service_id: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = RollbackDeployArgs.__new__(RollbackDeployArgs)

            if deploy_id is None and not opts.urn:
                raise TypeError("Missing required property 'deploy_id'")
            __props__.__dict__["deploy_id"] = deploy_id
            __props__.__dict__["service_id"] = service_id
            __props__.__dict__["commit"] = None
            __props__.__dict__["created_at"] = None
            __props__.__dict__["finished_at"] = None
            __props__.__dict__["image"] = None
            __props__.__dict__["status"] = None
            __props__.__dict__["trigger"] = None
            __props__.__dict__["updated_at"] = None
        super(RollbackDeploy, __self__).__init__(
            'render:services:RollbackDeploy',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None) -> 'RollbackDeploy':
        """
        Get an existing RollbackDeploy resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = RollbackDeployArgs.__new__(RollbackDeployArgs)

        __props__.__dict__["commit"] = None
        __props__.__dict__["created_at"] = None
        __props__.__dict__["deploy_id"] = None
        __props__.__dict__["finished_at"] = None
        __props__.__dict__["image"] = None
        __props__.__dict__["status"] = None
        __props__.__dict__["trigger"] = None
        __props__.__dict__["updated_at"] = None
        return RollbackDeploy(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter
    def commit(self) -> pulumi.Output[Optional['outputs.CommitProperties']]:
        return pulumi.get(self, "commit")

    @property
    @pulumi.getter(name="createdAt")
    def created_at(self) -> pulumi.Output[Optional[str]]:
        return pulumi.get(self, "created_at")

    @property
    @pulumi.getter(name="deployId")
    def deploy_id(self) -> pulumi.Output[str]:
        """
        The ID of the deploy to roll back to
        """
        return pulumi.get(self, "deploy_id")

    @property
    @pulumi.getter(name="finishedAt")
    def finished_at(self) -> pulumi.Output[Optional[str]]:
        return pulumi.get(self, "finished_at")

    @property
    @pulumi.getter
    def image(self) -> pulumi.Output[Optional['outputs.ImageProperties']]:
        """
        Image information used when creating the deploy. Not present for Git-backed deploys
        """
        return pulumi.get(self, "image")

    @property
    @pulumi.getter
    def status(self) -> pulumi.Output[Optional['Status']]:
        return pulumi.get(self, "status")

    @property
    @pulumi.getter
    def trigger(self) -> pulumi.Output[Optional['Trigger']]:
        return pulumi.get(self, "trigger")

    @property
    @pulumi.getter(name="updatedAt")
    def updated_at(self) -> pulumi.Output[Optional[str]]:
        return pulumi.get(self, "updated_at")

